@@ -4,9 +4,14 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2013-2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Copyright (C) 2013-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Volker Krause <volker.krause@kdab.com>
 
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  accordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
@@ -26,30 +31,73 @@
 
 #include <common/message.h>
 
+#include <QApplication>
 #include <QDataStream>
 #include <QDebug>
+#include <QStyle>
+#include <QStyleOptionViewItem>
 
 using namespace GammaRay;
 
+void (*RemoteModel::s_registerClientCallback)() = 0;
+
 RemoteModel::Node::~Node()
 {
   qDeleteAll(children);
 }
 
+void RemoteModel::Node::clearChildrenData()
+{
+  foreach (auto child, children) {
+    child->clearChildrenStructure();
+    child->data.clear();
+    child->flags.clear();
+    child->state.clear();
+  }
+}
+
+void RemoteModel::Node::clearChildrenStructure()
+{
+  qDeleteAll(children);
+  children.clear();
+  rowCount = -1;
+  columnCount = -1;
+}
+
+QVariant RemoteModel::s_emptyDisplayValue;
+QVariant RemoteModel::s_emptySizeHintValue;
 
 RemoteModel::RemoteModel(const QString &serverObject, QObject *parent) :
   QAbstractItemModel(parent),
+  m_pendingDataRequestsTimer(new QTimer(this)),
   m_serverObject(serverObject),
   m_myAddress(Protocol::InvalidObjectAddress),
   m_currentSyncBarrier(0),
-  m_targetSyncBarrier(0)
+  m_targetSyncBarrier(0),
+  m_proxyDynamicSortFilter(false),
+  m_proxyCaseSensitivity(Qt::CaseSensitive),
+  m_proxyKeyColumn(0)
 {
+  if (s_emptyDisplayValue.isNull()) {
+    s_emptyDisplayValue = tr("Loading...");
+#if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
+    QStyleOptionViewItemV4 opt;
+    opt.features |= QStyleOptionViewItemV4::HasDisplay;
+#else
+    QStyleOptionViewItem opt;
+    opt.features |= QStyleOptionViewItem::HasDisplay;
+#endif
+    opt.text = s_emptyDisplayValue.toString();
+    s_emptySizeHintValue = QApplication::style()->sizeFromContents(QStyle::CT_ItemViewItem, &opt, QSize(), Q_NULLPTR);
+  }
+
   m_root = new Node;
 
-  m_myAddress = Client::instance()->objectAddress(serverObject);
-  connect(Client::instance(), SIGNAL(objectRegistered(QString,Protocol::ObjectAddress)), SLOT(serverRegistered(QString,Protocol::ObjectAddress)));
-  connect(Client::instance(), SIGNAL(objectUnregistered(QString,Protocol::ObjectAddress)), SLOT(serverUnregistered(QString,Protocol::ObjectAddress)));
+  m_pendingDataRequestsTimer->setInterval(0);
+  m_pendingDataRequestsTimer->setSingleShot(true);
+  connect(m_pendingDataRequestsTimer, SIGNAL(timeout()), SLOT(doRequestDataAndFlags()));
 
+  registerClient(serverObject);
   connectToServer();
 }
 
@@ -99,9 +147,8 @@ int RemoteModel::rowCount(const QModelIndex &index) const
   if (node->rowCount < 0) {
     if (node->columnCount < 0) // not yet requested vs. in the middle of insertion
       requestRowColumnCount(index);
-    return 0;
   }
-  return node->rowCount;
+  return qMax(0, node->rowCount); // if requestRowColumnCount is synchronoous, ie. changes rowCount (as in simple unit test), returning 0 above would cause ModelTest to see inconsistent data
 }
 
 int RemoteModel::columnCount(const QModelIndex &index) const
@@ -127,17 +174,23 @@ QVariant RemoteModel::data(const QModelIndex &index, int role) const
   Q_ASSERT(node);
 
   const NodeStates state = stateForColumn(node, index.column());
-  if ((state & Outdated) && ((state & Loading) == 0)) {
-    requestDataAndFlags(index);
-  }
-
   if (role == LoadingState) {
     return QVariant::fromValue(state);
   }
 
+  // for size hint we don't want to trigger loading, as that's largely used for item view layouting
+  if (state & Empty) {
+    if (role == Qt::SizeHintRole)
+      return s_emptySizeHintValue;
+  }
+
+  if ((state & Outdated) && ((state & Loading) == 0)) {
+    requestDataAndFlags(index);
+  }
+
   if (state & Empty) { // still waiting for data
     if (role == Qt::DisplayRole)
-      return tr("Loading...");
+      return s_emptyDisplayValue;
     return QVariant();
   }
 
@@ -152,7 +205,7 @@ bool RemoteModel::setData(const QModelIndex& index, const QVariant& value, int r
 
   Message msg(m_myAddress, Protocol::ModelSetDataRequest);
   msg.payload() << Protocol::fromQModelIndex(index) << role << value;
-  Client::send(msg);
+  sendMessage(msg);
   return false;
 }
 
@@ -164,7 +217,7 @@ Qt::ItemFlags RemoteModel::flags(const QModelIndex& index) const
   const QHash<int, Qt::ItemFlags>::const_iterator it = node->flags.constFind(index.column());
   if (it == node->flags.constEnd()) {
     // default flags if we don't know better, otherwise we can't select into non-expanded branches
-    return Qt::ItemIsSelectable | Qt::ItemIsEnabled;
+    return index.isValid() ? Qt::ItemIsSelectable | Qt::ItemIsEnabled : Qt::NoItemFlags;
   }
   return it.value();
 }
@@ -180,6 +233,13 @@ QVariant RemoteModel::headerData(int section, Qt::Orientation orientation, int r
   return m_headers.value(orientation).value(section).value(role);
 }
 
+void RemoteModel::sort(int column, Qt::SortOrder order)
+{
+  Message msg(m_myAddress, Protocol::ModelSortRequest);
+  msg.payload() << (quint32)column << (quint32)order;
+  sendMessage(msg);
+}
+
 void RemoteModel::newMessage(const GammaRay::Message& msg)
 {
   if (!checkSyncBarrier(msg))
@@ -200,13 +260,16 @@ void RemoteModel::newMessage(const GammaRay::Message& msg)
         // the object may already have been invalidated.
         break;
       }
-      int rowCount, columnCount;
+      qint32 rowCount, columnCount;
       msg.payload() >> rowCount >> columnCount;
-      Q_ASSERT(rowCount >= 0 && columnCount >= 0);
-      if (rowCount == node->rowCount && columnCount == node->columnCount) {
+      // we get -1/-1 if we requested for an invalid index, e.g. due to not having processed
+      // all structure changes yet. This will automatically trigger a retry.
+      Q_ASSERT((rowCount >= 0 && columnCount >= 0) || (rowCount == -1 && columnCount == -1));
+      if (node->rowCount >= 0 || node->columnCount >= 0) {
         // This can happen in similar racy conditions as below, when we request the row/col count
         // for two different Node* at the same index (one was deleted inbetween and then the other
-        // was created). Anyhow, since the data is equal we can/should ignore it anyways.
+        // was created). We ignore the new data as the node it is intended for will request it again
+        // after processing all structure changes.
         break;
       }
 
@@ -217,15 +280,15 @@ void RemoteModel::newMessage(const GammaRay::Message& msg)
 
       const QModelIndex qmi = modelIndexForNode(node, 0);
 
-      if (columnCount) {
+      if (columnCount > 0) {
         beginInsertColumns(qmi, 0, columnCount - 1);
         node->columnCount = columnCount;
         endInsertColumns();
       } else {
-        node->columnCount = 0;
+        node->columnCount = columnCount;
       }
 
-      if (rowCount) {
+      if (rowCount > 0) {
         beginInsertRows(qmi, 0, rowCount - 1);
         node->children.reserve(rowCount);
         for (int i = 0; i < rowCount; ++i) {
@@ -236,28 +299,34 @@ void RemoteModel::newMessage(const GammaRay::Message& msg)
         node->rowCount = rowCount;
         endInsertRows();
       } else {
-        node->rowCount = 0;
+        node->rowCount = rowCount;
       }
       break;
     }
 
     case Protocol::ModelContentReply:
     {
-      Protocol::ModelIndex index;
-      msg.payload() >> index;
-      Node *node = nodeForIndex(index);
-      const NodeStates state = node ? stateForColumn(node, index.last().second) : NoState;
-      if ((state & Loading) == 0)
-        break; // we didn't ask for this, probably outdated response for a moved cell
-      typedef QHash<int, QVariant> ItemData;
-      ItemData itemData;
-      qint32 flags;
-      msg.payload() >> itemData >> flags;
-      node->data[index.last().second] = itemData;
-      node->flags[index.last().second] = static_cast<Qt::ItemFlags>(flags);
-      node->state.insert(index.last().second, state & ~(Loading | Empty | Outdated));
-      const QModelIndex qmi = modelIndexForNode(node, index.last().second);
-      emit dataChanged(qmi, qmi);
+      quint32 size;
+      msg.payload() >> size;
+      Q_ASSERT(size > 0);
+      for (quint32 i = 0; i < size; ++i) {
+        Protocol::ModelIndex index;
+        msg.payload() >> index;
+        Node *node = nodeForIndex(index);
+        const NodeStates state = node ? stateForColumn(node, index.last().second) : NoState;
+        typedef QHash<int, QVariant> ItemData;
+        ItemData itemData;
+        qint32 flags;
+        msg.payload() >> itemData >> flags;
+        if ((state & Loading) == 0)
+          continue; // we didn't ask for this, probably outdated response for a moved cell
+        node->data[index.last().second] = itemData;
+        node->flags[index.last().second] = static_cast<Qt::ItemFlags>(flags);
+        node->state.insert(index.last().second, state & ~(Loading | Empty | Outdated));
+        // TODO we could do some range compression here
+        const QModelIndex qmi = modelIndexForNode(node, index.last().second);
+        emit dataChanged(qmi, qmi);
+      }
       break;
     }
 
@@ -278,7 +347,8 @@ void RemoteModel::newMessage(const GammaRay::Message& msg)
     case Protocol::ModelContentChanged:
     {
       Protocol::ModelIndex beginIndex, endIndex;
-      msg.payload() >> beginIndex >> endIndex;
+      QVector<int> roles;
+      msg.payload() >> beginIndex >> endIndex >> roles;
       Node *node = nodeForIndex(beginIndex);
       if (!node || node == m_root)
         break;
@@ -299,7 +369,11 @@ void RemoteModel::newMessage(const GammaRay::Message& msg)
       const QModelIndex qmiBegin = modelIndexForNode(node, beginIndex.last().second);
       const QModelIndex qmiEnd = qmiBegin.sibling(endIndex.last().first, endIndex.last().second);
 
+#if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
       emit dataChanged(qmiBegin, qmiEnd);
+#else
+      emit dataChanged(qmiBegin, qmiEnd, roles);
+#endif
       break;
     }
 
@@ -384,10 +458,62 @@ void RemoteModel::newMessage(const GammaRay::Message& msg)
     case Protocol::ModelColumnsAdded:
     case Protocol::ModelColumnsMoved:
     case Protocol::ModelColumnsRemoved:
-    case Protocol::ModelLayoutChanged:
     {
       // TODO
       qWarning() << Q_FUNC_INFO << "not implemented yet" << msg.type() << m_serverObject;
+      clear();
+      break;
+    }
+
+    case Protocol::ModelLayoutChanged:
+    {
+      QVector<Protocol::ModelIndex> parents;
+      quint32 hint;
+      msg.payload() >> parents >> hint;
+
+      if (parents.isEmpty()) { // everything changed (or Qt4)
+        emit layoutAboutToBeChanged();
+        foreach (const auto &persistentIndex, persistentIndexList()) {
+          changePersistentIndex(persistentIndex, QModelIndex());
+        }
+        if (hint == 0)
+          m_root->clearChildrenStructure();
+        else
+          m_root->clearChildrenData();
+        emit layoutChanged();
+        break;
+      }
+
+      QVector<Node*> parentNodes;
+      parentNodes.reserve(parents.size());
+      foreach (const auto &p, parents) {
+        auto node = nodeForIndex(p);
+        if (!node)
+          continue;
+        parentNodes.push_back(node);
+      }
+      if (parentNodes.isEmpty())
+        break; // no currently loaded node changed, nothing to do
+
+      emit layoutAboutToBeChanged(); // TODO Qt5 support with exact sub-trees
+      foreach (const auto &persistentIndex, persistentIndexList()) {
+        auto persistentNode = nodeForIndex(persistentIndex);
+        Q_ASSERT(persistentNode);
+        foreach (auto node, parentNodes) {
+          if (!isAncestor(node, persistentNode))
+            continue;
+          changePersistentIndex(persistentIndex, QModelIndex());
+          break;
+        }
+      }
+      foreach (auto node, parentNodes) {
+        if (hint == 0)
+          node->clearChildrenStructure();
+        else
+          node->clearChildrenData();
+      }
+      emit layoutChanged(); // TODO Qt5 support with exact sub-trees
+      break;
     }
 
     case Protocol::ModelReset:
@@ -428,7 +554,7 @@ RemoteModel::Node* RemoteModel::nodeForIndex(const Protocol::ModelIndex &index)
   for (int i = 0; i < index.size(); ++i) {
     if (node->children.size() <= index[i].first)
       return 0;
-    node = node->children[index[i].first];
+    node = node->children.at(index[i].first);
   }
   return node;
 }
@@ -441,6 +567,20 @@ QModelIndex RemoteModel::modelIndexForNode(Node* node, int column) const
   return createIndex(node->parent->children.indexOf(node), column, node);
 }
 
+bool RemoteModel::isAncestor(RemoteModel::Node* ancestor, RemoteModel::Node* child) const
+{
+  Q_ASSERT(ancestor);
+  Q_ASSERT(child);
+  Q_ASSERT(m_root);
+
+  if (child == m_root)
+    return false;
+  Q_ASSERT(child->parent);
+  if (child->parent == ancestor)
+    return true;
+  return isAncestor(ancestor, child->parent);
+}
+
 RemoteModel::NodeStates RemoteModel::stateForColumn(RemoteModel::Node* node, int columnIndex) const
 {
   Q_ASSERT(node);
@@ -462,7 +602,7 @@ void RemoteModel::requestRowColumnCount(const QModelIndex &index) const
 
   Message msg(m_myAddress, Protocol::ModelRowColumnCountRequest);
   msg.payload() << Protocol::fromQModelIndex(index);
-  Client::send(msg);
+  sendMessage(msg);
 }
 
 void RemoteModel::requestDataAndFlags(const QModelIndex& index) const
@@ -477,29 +617,46 @@ void RemoteModel::requestDataAndFlags(const QModelIndex& index) const
 
   node->state.insert(index.column(), state | Loading); // mark pending request
 
+  m_pendingDataRequests.push_back(Protocol::fromQModelIndex(index));
+  if (m_pendingDataRequests.size() > 100) {
+    m_pendingDataRequestsTimer->stop();
+    doRequestDataAndFlags();
+  } else {
+    m_pendingDataRequestsTimer->start();
+  }
+}
+
+void RemoteModel::doRequestDataAndFlags() const
+{
+  Q_ASSERT(!m_pendingDataRequests.isEmpty());
   Message msg(m_myAddress, Protocol::ModelContentRequest);
-  msg.payload() << Protocol::fromQModelIndex(index);
-  Client::send(msg);
+  msg.payload() << quint32(m_pendingDataRequests.size());
+  foreach (const auto &index, m_pendingDataRequests)
+    msg.payload() << index;
+  m_pendingDataRequests.clear();
+  sendMessage(msg);
 }
 
 void RemoteModel::requestHeaderData(Qt::Orientation orientation, int section) const
 {
   Q_ASSERT(section >= 0);
   Q_ASSERT(!m_headers.value(orientation).contains(section));
-  m_headers[orientation][section][Qt::DisplayRole] = tr("Loading...");
+  m_headers[orientation][section][Qt::DisplayRole] = s_emptyDisplayValue;
 
   Message msg(m_myAddress, Protocol::ModelHeaderRequest);
   msg.payload() << qint8(orientation) << qint32(section);
-  Client::send(msg);
+  sendMessage(msg);
 }
 
 void RemoteModel::clear()
 {
   beginResetModel();
 
-  Message msg(m_myAddress, Protocol::ModelSyncBarrier);
-  msg.payload() << ++m_targetSyncBarrier;
-  Client::send(msg);
+  if (isConnected()) {
+    Message msg(m_myAddress, Protocol::ModelSyncBarrier);
+    msg.payload() << ++m_targetSyncBarrier;
+    sendMessage(msg);
+  }
 
   delete m_root;
   m_root = new Node;
@@ -513,7 +670,8 @@ void RemoteModel::connectToServer()
     return;
 
   beginResetModel();
-  Client::instance()->registerForObject(m_myAddress, this, "newMessage");
+  Client::instance()->registerObject(m_serverObject, this);
+  Client::instance()->registerMessageHandler(m_myAddress, this, "newMessage");
   endResetModel();
 }
 
@@ -534,7 +692,7 @@ void RemoteModel::resetLoadingState(RemoteModel::Node* node, int startRow) const
 
   Q_ASSERT(node->children.size() == node->rowCount);
   for (int row = startRow; row < node->rowCount; ++row) {
-    Node *child = node->children[row];
+    Node *child = node->children.at(row);
     for (QHash<int, NodeStates>::iterator it = child->state.begin(); it != child->state.end(); ++it) {
       if (it.value() & Loading)
         it.value() = it.value() & ~Loading;
@@ -595,6 +753,8 @@ void RemoteModel::doMoveRows(RemoteModel::Node* sourceParentNode, int sourceStar
 {
   Q_ASSERT(sourceParentNode->rowCount == sourceParentNode->children.size());
   Q_ASSERT(destParentNode->rowCount == destParentNode->children.size());
+  Q_ASSERT(sourceEnd >= sourceStart);
+  Q_ASSERT(sourceParentNode->rowCount > sourceEnd);
 
   const int destEnd = destStart + sourceEnd - sourceStart;
   const int amount = sourceEnd - sourceStart + 1;
@@ -629,3 +789,71 @@ void RemoteModel::doMoveRows(RemoteModel::Node* sourceParentNode, int sourceStar
   resetLoadingState(sourceParentNode, sourceStart);
   resetLoadingState(destParentNode, destEnd);
 }
+
+void RemoteModel::registerClient(const QString &serverObject)
+{
+  if (Q_UNLIKELY(s_registerClientCallback)) { // called from ctor, so we can't use virtuals here
+    s_registerClientCallback();
+    return;
+  }
+  m_myAddress = Endpoint::instance()->objectAddress(serverObject);
+  connect(Endpoint::instance(), SIGNAL(objectRegistered(QString,Protocol::ObjectAddress)), SLOT(serverRegistered(QString,Protocol::ObjectAddress)));
+  connect(Endpoint::instance(), SIGNAL(objectUnregistered(QString,Protocol::ObjectAddress)), SLOT(serverUnregistered(QString,Protocol::ObjectAddress)));
+}
+
+void RemoteModel::sendMessage(const Message& msg) const
+{
+  Endpoint::send(msg);
+}
+
+bool RemoteModel::proxyDynamicSortFilter() const
+{
+  return m_proxyDynamicSortFilter;
+}
+
+void RemoteModel::setProxyDynamicSortFilter(bool dynamicSortFilter)
+{
+  if (m_proxyDynamicSortFilter == dynamicSortFilter)
+    return;
+  m_proxyDynamicSortFilter = dynamicSortFilter;
+  emit proxyDynamicSortFilterChanged();
+}
+
+Qt::CaseSensitivity RemoteModel::proxyFilterCaseSensitivity() const
+{
+  return m_proxyCaseSensitivity;
+}
+
+void RemoteModel::setProxyFilterCaseSensitivity(Qt::CaseSensitivity caseSensitivity)
+{
+  if (m_proxyCaseSensitivity == caseSensitivity)
+    return;
+  m_proxyCaseSensitivity = caseSensitivity;
+  emit proxyFilterCaseSensitivityChanged();
+}
+
+int RemoteModel::proxyFilterKeyColumn() const
+{
+  return m_proxyKeyColumn;
+}
+
+void RemoteModel::setProxyFilterKeyColumn(int column)
+{
+  if (m_proxyKeyColumn == column)
+    return;
+  m_proxyKeyColumn = column;
+  emit proxyFilterKeyColumnChanged();
+}
+
+QRegExp RemoteModel::proxyFilterRegExp() const
+{
+  return m_proxyFilterRegExp;
+}
+
+void RemoteModel::setProxyFilterRegExp(const QRegExp& regExp)
+{
+  if (m_proxyFilterRegExp == regExp)
+    return;
+  m_proxyFilterRegExp = regExp;
+  emit proxyFilterRegExpChanged();
+}
