@@ -4,9 +4,14 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2013-2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Copyright (C) 2013-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Volker Krause <volker.krause@kdab.com>
 
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  acuordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
@@ -27,14 +32,17 @@
 #include "gammaray_common_export.h"
 #include "protocol.h"
 
+#include <QMetaMethod>
 #include <QObject>
 #include <QPointer>
 
 class QIODevice;
+class QUrl;
 
 namespace GammaRay {
 
 class Message;
+class PropertySyncer;
 
 /** @brief Network protocol endpoint.
  *
@@ -102,10 +110,22 @@ public:
   virtual bool isRemoteClient() const = 0;
 
   /**
-   * Returns the address of the server, in case you need to connect to a different service there
+   * Returns the listening address of the server, in case you need to connect to a different service there
    * (such as the web inspector server).
    */
-  virtual QString serverAddress() const = 0;
+  virtual QUrl serverAddress() const = 0;
+
+  /** Register the slot @p messageHandlerName on @p receiver as the handler for messages to/from @p objectAddress.
+   *  @see dispatchMessage()
+   */
+  virtual void registerMessageHandler(Protocol::ObjectAddress objectAddress, QObject *receiver, const char* messageHandlerName);
+
+  /** Unregister the message handler for @p objectAddress. */
+  virtual void unregisterMessageHandler(Protocol::ObjectAddress objectAddress);
+
+public slots:
+  /** Convenience overload of send(), to directly send message delivered via signals. */
+  void sendMessage(const GammaRay::Message &msg);
 
 signals:
   /** Emitted when we lost the connection to the other endpoint. */
@@ -129,17 +149,9 @@ protected:
   virtual void messageReceived(const Message &msg) = 0;
 
   /** Call this when learning about a new object <-> address mapping. */
-  void registerObjectInternal(const QString &objectName, Protocol::ObjectAddress objectAddress);
+  void addObjectNameAddressMapping(const QString &objectName, Protocol::ObjectAddress objectAddress);
   /** Call this when learning about a dissolved object <-> address mapping. */
-  void unregisterObjectInternal(const QString& objectName);
-
-  /** Register the slot @p messageHandlerName on @p receiver as the handler for messages to/from @p objectAddress.
-   *  @see dispatchMessage()
-   */
-  void registerMessageHandlerInternal(Protocol::ObjectAddress objectAddress, QObject *receiver, const char* messageHandlerName);
-
-  /** Unregister the message handler for @p objectAddress. */
-  void unregisterMessageHandlerInternal(Protocol::ObjectAddress objectAddress);
+  void removeObjectNameAddressMapping(const QString& objectName);
 
   /** Called when the current handler of the object identified by @p objectAddress has been destroyed. */
   virtual void handlerDestroyed(Protocol::ObjectAddress objectAddress, const QString &objectName) = 0;
@@ -163,6 +175,8 @@ protected:
    */
   void invokeObjectLocal(QObject *object, const char *method, const QVariantList &args) const;
 
+  PropertySyncer *m_propertySyncer;
+
 private slots:
   void readyRead();
   void connectionClosed();
@@ -184,9 +198,8 @@ private:
     QObject *object;
 
     // custom message handling support
-    // TODO: obsolete this
     QObject *receiver;
-    QByteArray messageHandler;
+    QMetaMethod messageHandler;
   };
 
   /** Inserts @p oi into all maps. */
