@@ -4,9 +4,14 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2013-2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Copyright (C) 2013-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Volker Krause <volker.krause@kdab.com>
 
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  acuordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
@@ -45,7 +50,7 @@ class GammaRay::MethodArgumentPrivate : public QSharedData
     ~MethodArgumentPrivate()
     {
       if (data)
-        QMetaType::destroy(value.type(), data);
+        QMetaType::destroy(value.userType(), data);
     }
 
     QVariant value;
@@ -88,13 +93,13 @@ MethodArgument& MethodArgument::operator=(const MethodArgument& other)
 MethodArgument::operator QGenericArgument() const
 {
   if (!d->unwrapVariant) {
-    return QGenericArgument(d->name.data(), &d->value);
+    return QGenericArgument(d->name.constData(), &d->value);
   }
 
   if (d->value.isValid()) {
     d->data = QMetaType::construct(d->value.userType(), d->value.constData());
     Q_ASSERT(d->data);
-    return QGenericArgument(d->name.data(), d->data);
+    return QGenericArgument(d->name.constData(), d->data);
   }
 
   return QGenericArgument();
