@@ -4,9 +4,14 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2013-2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Copyright (C) 2013-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Volker Krause <volker.krause@kdab.com>
 
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  acuordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
@@ -21,7 +26,7 @@
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include "config-gammaray.h"
+#include <config-gammaray.h>
 #include "paths.h"
 
 #include <QCoreApplication>
@@ -57,10 +62,15 @@ void setRelativeRootPath(const char* relativeRootPath)
 
 QString probePath(const QString& probeABI)
 {
+#ifndef GAMMARAY_INSTALL_QT_LAYOUT
   return rootPath() + QDir::separator()
     + QLatin1String(GAMMARAY_PLUGIN_INSTALL_DIR) + QDir::separator()
     + QLatin1String(GAMMARAY_PLUGIN_VERSION) + QDir::separator()
     + probeABI;
+#else
+  Q_UNUSED(probeABI);
+  return rootPath() + QDir::separator() + QLatin1String(GAMMARAY_PROBE_INSTALL_DIR);
+#endif
 }
 
 QString binPath()
@@ -75,8 +85,39 @@ QString libexecPath()
 
 QString currentProbePath()
 {
-  return probePath(GAMMARAY_PROBE_ABI);
+  return probePath(QStringLiteral(GAMMARAY_PROBE_ABI));
+}
+
+QString currentPluginsPath()
+{
+#ifndef GAMMARAY_INSTALL_QT_LAYOUT
+  return currentProbePath();
+#else
+  return rootPath() + QDir::separator() + QStringLiteral(GAMMARAY_PLUGIN_INSTALL_DIR);
+#endif
+
+}
+
+QString libraryExtension()
+{
+#ifdef Q_OS_WIN
+  return QStringLiteral(".dll");
+#elif defined(Q_OS_MAC)
+  return QStringLiteral(".dylib");
+#else
+  return QStringLiteral(".so");
+#endif
+}
+
+QString pluginExtension()
+{
+#ifdef Q_OS_MAC
+  return QStringLiteral(".so");
+#else
+  return libraryExtension();
+#endif
 }
 
 }
+
 }
