@@ -2,9 +2,14 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2013-2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Copyright (C) 2013-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Milian Wolff <milian.wolff@kdab.com>
 
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  acuordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
@@ -22,6 +27,8 @@
 #ifndef GAMMARAY_SETTEMPVALUE_H
 #define GAMMARAY_SETTEMPVALUE_H
 
+#include <qglobal.h>
+
 namespace GammaRay {
 
 /**
@@ -46,6 +53,7 @@ namespace Util {
     {
       obj = oldValue;
     }
+    Q_DISABLE_COPY(SetTempValue)
     T& obj;
     T oldValue;
   };
