@@ -4,9 +4,14 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2011-2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Copyright (C) 2011-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Volker Krause <volker.krause@kdab.com>
 
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  accordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
@@ -26,16 +31,22 @@
 
 #include <common/metatypedeclarations.h>
 
+#include <QAbstractItemModel>
 #include <QAbstractSocket>
+#include <QCoreApplication>
 #include <QFile>
+#include <QFont>
 #include <QNetworkProxy>
 #include <QObject>
 #include <QPalette>
 #include <QPen>
 #include <QSocketNotifier>
+#include <QSortFilterProxyModel>
 #include <QTcpServer>
+#include <QThread>
 
 #if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
+#include <QGuiApplication>
 #include <QOpenGLContext>
 #include <QOpenGLShader>
 #include <QScreen>
@@ -82,12 +93,27 @@ void MetaObjectRepository::initBuiltInTypes()
   initOpenGLTypes();
 }
 
+#if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
+Q_DECLARE_METATYPE(QThread*)
+#endif
+Q_DECLARE_METATYPE(QThread::Priority)
+
 void MetaObjectRepository::initQObjectTypes()
 {
   MetaObject *mo = 0;
   MO_ADD_METAOBJECT0(QObject);
   MO_ADD_PROPERTY_RO(QObject, QObject*, parent);
   MO_ADD_PROPERTY_RO(QObject, bool, signalsBlocked); // TODO setter has non-void return type
+  MO_ADD_PROPERTY_RO(QObject, QThread*, thread);
+
+  MO_ADD_METAOBJECT1(QThread, QObject)
+  MO_ADD_PROPERTY_RO(QThread, bool, isFinished);
+#if QT_VERSION >= QT_VERSION_CHECK(5, 2, 0)
+  MO_ADD_PROPERTY_RO(QThread, bool, isInterruptionRequested);
+#endif
+  MO_ADD_PROPERTY_RO(QThread, bool, isRunning);
+  MO_ADD_PROPERTY   (QThread, QThread::Priority, priority, setPriority);
+  MO_ADD_PROPERTY   (QThread, uint, stackSize, setStackSize);
 
   MO_ADD_METAOBJECT0(QPaintDevice);
   MO_ADD_PROPERTY_RO(QPaintDevice, int, colorCount);
@@ -98,6 +124,52 @@ void MetaObjectRepository::initQObjectTypes()
   MO_ADD_PROPERTY_RO(QPaintDevice, int, physicalDpiX);
   MO_ADD_PROPERTY_RO(QPaintDevice, int, physicalDpiY);
   MO_ADD_PROPERTY_RO(QPaintDevice, int, widthMM);
+
+  MO_ADD_METAOBJECT1(QCoreApplication, QObject);
+  MO_ADD_PROPERTY_ST(QCoreApplication, QString, applicationDirPath);
+  MO_ADD_PROPERTY_ST(QCoreApplication, QString, applicationFilePath);
+  MO_ADD_PROPERTY_ST(QCoreApplication, qint64, applicationPid);
+  MO_ADD_PROPERTY_ST(QCoreApplication, QStringList, arguments);
+  MO_ADD_PROPERTY_ST(QCoreApplication, bool, closingDown);
+  MO_ADD_PROPERTY_ST(QCoreApplication, bool, hasPendingEvents);
+#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
+  MO_ADD_PROPERTY_ST(QCoreApplication, bool, isQuitLockEnabled);
+#endif
+#if QT_VERSION >= QT_VERSION_CHECK(5, 3, 0)
+  MO_ADD_PROPERTY_ST(QCoreApplication, bool, isSetuidAllowed);
+#endif
+  MO_ADD_PROPERTY_ST(QCoreApplication, QStringList, libraryPaths);
+  MO_ADD_PROPERTY_ST(QCoreApplication, bool, startingUp);
+
+#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
+  MO_ADD_METAOBJECT1(QGuiApplication, QCoreApplication);
+  MO_ADD_PROPERTY_ST(QGuiApplication, Qt::ApplicationState, applicationState);
+  MO_ADD_PROPERTY_ST(QGuiApplication, bool, desktopSettingsAware);
+  MO_ADD_PROPERTY_RO(QGuiApplication, qreal, devicePixelRatio);
+  MO_ADD_PROPERTY_ST(QGuiApplication, QObject*, focusObject);
+  MO_ADD_PROPERTY_ST(QGuiApplication, QWindow*, focusWindow);
+  MO_ADD_PROPERTY_ST(QGuiApplication, QFont, font);
+  MO_ADD_PROPERTY_ST(QGuiApplication, bool, isLeftToRight);
+  MO_ADD_PROPERTY_ST(QGuiApplication, bool, isRightToLeft);
+  MO_ADD_PROPERTY_ST(QGuiApplication, QPalette, palette);
+  MO_ADD_PROPERTY_ST(QGuiApplication, QScreen*, primaryScreen);
+#ifndef QT_NO_SESSIONMANAGER
+  MO_ADD_PROPERTY_RO(QGuiApplication, bool, isSavingSession);
+  MO_ADD_PROPERTY_RO(QGuiApplication, bool, isSessionRestored);
+  MO_ADD_PROPERTY_RO(QGuiApplication, QString, sessionId);
+  MO_ADD_PROPERTY_RO(QGuiApplication, QString, sessionKey);
+#endif
+#endif
+
+  MO_ADD_METAOBJECT1(QAbstractItemModel, QObject);
+  MO_ADD_PROPERTY_RO(QAbstractItemModel, QStringList, mimeTypes);
+#if QT_VERSION >= QT_VERSION_CHECK(5, 5, 0)
+  MO_ADD_PROPERTY_RO(QAbstractItemModel, Qt::DropActions, supportedDragActions);
+  MO_ADD_PROPERTY_RO(QAbstractItemModel, Qt::DropActions, supportedDropActions);
+#endif
+  MO_ADD_METAOBJECT1(QAbstractProxyModel, QAbstractItemModel);
+  MO_ADD_METAOBJECT1(QSortFilterProxyModel, QAbstractProxyModel);
+  MO_ADD_PROPERTY_RO(QSortFilterProxyModel, Qt::SortOrder, sortOrder);
 }
 
 
@@ -117,6 +189,9 @@ Q_DECLARE_METATYPE(QAbstractSocket::SocketState)
 Q_DECLARE_METATYPE(QNetworkProxy)
 #endif
 #endif
+#if QT_VERSION < QT_VERSION_CHECK(5, 5, 0)
+Q_DECLARE_METATYPE(Qt::SortOrder)
+#endif
 
 void MetaObjectRepository::initIOTypes()
 {
@@ -213,6 +288,8 @@ void MetaObjectRepository::initNetworkTypes()
 void MetaObjectRepository::initGuiTypes()
 {
 #if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
+  qRegisterMetaType<QScreen*>();
+
   MetaObject *mo = 0;
   MO_ADD_METAOBJECT0(QSurface);
   MO_ADD_PROPERTY_RO(QSurface, QSurfaceFormat, format);
@@ -248,10 +325,6 @@ void MetaObjectRepository::initGuiTypes()
 #endif
 }
 
-#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
-Q_DECLARE_METATYPE(QOpenGLShader::ShaderType)
-#endif
-
 void MetaObjectRepository::initOpenGLTypes()
 {
 #if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
@@ -267,14 +340,16 @@ void MetaObjectRepository::initOpenGLTypes()
   MO_ADD_PROPERTY_RO(QOpenGLShaderProgram, bool, isLinked);
   MO_ADD_PROPERTY_RO(QOpenGLShaderProgram, QString, log);
 #if QT_VERSION >= QT_VERSION_CHECK(5, 1, 0)
-  MO_ADD_PROPERTY_RO(QOpenGLShaderProgram, int, maxGeometryOutputVertices);
+// FIXME calling this asserts in debug builds of some newer Qt versions
+//   MO_ADD_PROPERTY_RO(QOpenGLShaderProgram, int, maxGeometryOutputVertices);
   MO_ADD_PROPERTY   (QOpenGLShaderProgram, int, patchVertexCount, setPatchVertexCount);
 #endif
   MO_ADD_PROPERTY_RO(QOpenGLShaderProgram, uint, programId);
 
   MO_ADD_METAOBJECT1(QOpenGLContext, QObject);
   MO_ADD_PROPERTY_RO(QOpenGLContext, uint, defaultFramebufferObject);
-  MO_ADD_PROPERTY_RO(QOpenGLContext, QSet<QByteArray>, extensions);
+  // crashes if context isn't current
+//   MO_ADD_PROPERTY_RO(QOpenGLContext, QSet<QByteArray>, extensions);
   MO_ADD_PROPERTY_RO(QOpenGLContext, QSurfaceFormat, format);
   MO_ADD_PROPERTY_RO(QOpenGLContext, bool, isValid);
   MO_ADD_PROPERTY_RO(QOpenGLContext, QScreen*, screen);
@@ -302,8 +377,8 @@ MetaObject *MetaObjectRepository::metaObject(const QString &typeName) const
   QString typeName_ = typeName;
   typeName_.remove('*');
   typeName_.remove('&');
-  typeName_.remove("const ");
-  typeName_.remove(" const");
+  typeName_.remove(QStringLiteral("const "));
+  typeName_.remove(QStringLiteral(" const"));
   typeName_.remove(' ');
   return m_metaObjects.value(typeName_);
 }
