@@ -4,9 +4,14 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2011-2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Copyright (C) 2011-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Volker Krause <volker.krause@kdab.com>
 
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  accordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
@@ -70,6 +75,7 @@ class GAMMARAY_CORE_EXPORT MetaObjectRepository
     MetaObjectRepository();
 
   private:
+    Q_DISABLE_COPY(MetaObjectRepository)
     void initBuiltInTypes();
     void initQObjectTypes();
     void initIOTypes();
@@ -85,8 +91,8 @@ class GAMMARAY_CORE_EXPORT MetaObjectRepository
 }
 ///@cond internal
 #define MO_ADD_BASECLASS(Base) \
-  Q_ASSERT(GammaRay::MetaObjectRepository::instance()->hasMetaObject(QLatin1String(#Base))); \
-  mo->addBaseClass(GammaRay::MetaObjectRepository::instance()->metaObject(QLatin1String(#Base)));
+  Q_ASSERT(GammaRay::MetaObjectRepository::instance()->hasMetaObject(QStringLiteral(#Base))); \
+  mo->addBaseClass(GammaRay::MetaObjectRepository::instance()->metaObject(QStringLiteral(#Base)));
 ///@endcond
 
 /** Register @p Class with the MetaObjectRepository.
@@ -94,7 +100,7 @@ class GAMMARAY_CORE_EXPORT MetaObjectRepository
  */
 #define MO_ADD_METAOBJECT0(Class) \
   mo = new GammaRay::MetaObjectImpl<Class>; \
-  mo->setClassName(QLatin1String(#Class)); \
+  mo->setClassName(QStringLiteral(#Class)); \
   GammaRay::MetaObjectRepository::instance()->addMetaObject(mo);
 
 /** Register @p Class with the MetaObjectRepository.
@@ -102,7 +108,7 @@ class GAMMARAY_CORE_EXPORT MetaObjectRepository
  */
 #define MO_ADD_METAOBJECT1(Class, Base1) \
   mo = new GammaRay::MetaObjectImpl<Class, Base1>; \
-  mo->setClassName(QLatin1String(#Class)); \
+  mo->setClassName(QStringLiteral(#Class)); \
   MO_ADD_BASECLASS(Base1) \
   GammaRay::MetaObjectRepository::instance()->addMetaObject(mo);
 
@@ -111,7 +117,7 @@ class GAMMARAY_CORE_EXPORT MetaObjectRepository
  */
 #define MO_ADD_METAOBJECT2(Class, Base1, Base2) \
   mo = new GammaRay::MetaObjectImpl<Class, Base1, Base2>; \
-  mo->setClassName(QLatin1String(#Class)); \
+  mo->setClassName(QStringLiteral(#Class)); \
   MO_ADD_BASECLASS(Base1) \
   MO_ADD_BASECLASS(Base2) \
   GammaRay::MetaObjectRepository::instance()->addMetaObject(mo);
@@ -119,7 +125,7 @@ class GAMMARAY_CORE_EXPORT MetaObjectRepository
 /** Register a read/write property for class @p Class. */
 #define MO_ADD_PROPERTY(Class, Type, Getter, Setter) \
   mo->addProperty(new GammaRay::MetaPropertyImpl<Class, Type>( \
-    QLatin1String(#Getter), \
+    #Getter, \
     &Class::Getter, \
     static_cast<void (Class::*)(Type)>(&Class::Setter)) \
   );
@@ -127,7 +133,7 @@ class GAMMARAY_CORE_EXPORT MetaObjectRepository
 /** Register a read/write property for class @p Class with a type that is passed as const reference. */
 #define MO_ADD_PROPERTY_CR(Class, Type, Getter, Setter) \
   mo->addProperty(new GammaRay::MetaPropertyImpl<Class, Type, const Type&>( \
-    QLatin1String(#Getter), \
+    #Getter, \
     &Class::Getter, \
     static_cast<void (Class::*)(const Type&)>(&Class::Setter)) \
   );
@@ -135,7 +141,13 @@ class GAMMARAY_CORE_EXPORT MetaObjectRepository
 /** Register a read-only property for class @p Class. */
 #define MO_ADD_PROPERTY_RO(Class, Type, Getter) \
   mo->addProperty(new GammaRay::MetaPropertyImpl<Class, Type>( \
-    QLatin1String(#Getter), \
+    #Getter, \
+    &Class::Getter));
+
+/** Register a static property for class @p Class. */
+#define MO_ADD_PROPERTY_ST(Class, Type, Getter) \
+  mo->addProperty(new GammaRay::MetaStaticPropertyImpl<Class, Type>( \
+    #Getter, \
     &Class::Getter));
 
 #endif // GAMMARAY_METAOBJECTREPOSITORY_H
