@@ -4,9 +4,14 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2011-2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Copyright (C) 2011-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Volker Krause <volker.krause@kdab.com>
 
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  accordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
@@ -24,12 +29,11 @@
 #include "metaproperty.h"
 #include "metaobject.h"
 
-#include <QString>
 #include <QVariant>
 
 using namespace GammaRay;
 
-MetaProperty::MetaProperty() : m_class(0)
+MetaProperty::MetaProperty(const char *name) : m_class(0), m_name(name)
 {
 }
 
@@ -37,6 +41,18 @@ MetaProperty::~MetaProperty()
 {
 }
 
+const char* MetaProperty::name() const
+{
+  return m_name;
+}
+
+void MetaProperty::setValue(void* object, const QVariant& value)
+{
+  Q_UNUSED(object);
+  Q_UNUSED(value);
+  Q_ASSERT(isReadOnly()); // otherwise sub-class should have implement this...
+}
+
 MetaObject *MetaProperty::metaObject() const
 {
   Q_ASSERT(m_class);
