@@ -4,9 +4,14 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2010-2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Copyright (C) 2010-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Volker Krause <volker.krause@kdab.com>
 
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  accordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
@@ -36,7 +41,7 @@ class MultiSignalMapperPrivate : public QObject
     explicit MultiSignalMapperPrivate(MultiSignalMapper* parent) : QObject(parent), q(parent) {}
     ~MultiSignalMapperPrivate() {}
 
-    int qt_metacall(QMetaObject::Call call, int methodId, void** args)
+    int qt_metacall(QMetaObject::Call call, int methodId, void** args) Q_DECL_OVERRIDE
     {
       methodId = QObject::qt_metacall(call, methodId, args);
       if (methodId < 0)
@@ -44,9 +49,6 @@ class MultiSignalMapperPrivate : public QObject
 
       if (call == QMetaObject::InvokeMetaMethod) {
         Q_ASSERT(sender());
-#if QT_VERSION >= 0x040800
-        Q_ASSERT(senderSignalIndex() == methodId);
-#endif
         const QVector<QVariant> v = convertArguments(sender(), methodId, args);
         emit q->signalEmitted(sender(), methodId, v);
         return -1; // indicates we handled the call
