@@ -4,9 +4,14 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2010-2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Copyright (C) 2010-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Volker Krause <volker.krause@kdab.com>
 
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  accordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
@@ -34,6 +39,7 @@
 #ifndef GAMMARAY_OBJECTTYPEFILTERPROXYMODEL_H
 #define GAMMARAY_OBJECTTYPEFILTERPROXYMODEL_H
 
+#include "gammaray_core_export.h"
 #include "objectmodelbase.h"
 
 #include <QSortFilterProxyModel>
@@ -43,17 +49,15 @@ namespace GammaRay {
 /**
  * @brief A QSortFilterProxyModel for generic Objects.
  */
-class ObjectFilterProxyModelBase : public QSortFilterProxyModel
+class GAMMARAY_CORE_EXPORT ObjectFilterProxyModelBase : public QSortFilterProxyModel
 {
+  Q_OBJECT
   public:
     /**
      * Constructor.
      * @param parent is the parent object for this instance.
      */
-    explicit ObjectFilterProxyModelBase(QObject *parent = 0) : QSortFilterProxyModel(parent)
-    {
-      setDynamicSortFilter(true);
-    }
+    explicit ObjectFilterProxyModelBase(QObject *parent = Q_NULLPTR);
 
   protected:
     /**
@@ -63,20 +67,7 @@ class ObjectFilterProxyModelBase : public QSortFilterProxyModel
      * @return true if the item in the row can be included in the model;
      *         otherwise returns false.
      */
-    bool filterAcceptsRow(int source_row, const QModelIndex &source_parent) const
-    {
-      const QModelIndex source_index = sourceModel()->index(source_row, 0, source_parent);
-      if (!source_index.isValid()) {
-        return false;
-      }
-
-      QObject *obj = source_index.data(ObjectModel::ObjectRole).value<QObject*>();
-      if (!obj || !filterAcceptsObject(obj)) {
-        return false;
-      }
-
-      return QSortFilterProxyModel::filterAcceptsRow(source_row, source_parent);
-    }
+    bool filterAcceptsRow(int source_row, const QModelIndex &source_parent) const Q_DECL_OVERRIDE;
 
     /**
      * Determines if the specified QObject can be included in the model.
@@ -103,7 +94,7 @@ class ObjectTypeFilterProxyModel : public ObjectFilterProxyModelBase
     }
 
   protected:
-    virtual bool filterAcceptsObject(QObject *object) const
+    bool filterAcceptsObject(QObject *object) const Q_DECL_OVERRIDE
     {
       return qobject_cast<T*>(object);
     }
