@@ -4,10 +4,15 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2010-2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Copyright (C) 2010-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Volker Krause <volker.krause@kdab.com>
   Author: Stephen Kelly <stephen.kelly@kdab.com>
 
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  accordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
@@ -23,24 +28,27 @@
 */
 //krazy:excludeall=null,captruefalse,staticobjects
 
+#include <config-gammaray.h>
+
 #include "probe.h"
 #include "objectlistmodel.h"
 #include "objecttreemodel.h"
 #include "metaobjecttreemodel.h"
-#include "connectionmodel.h"
 #include "toolmodel.h"
-#include "readorwritelocker.h"
 #include "probesettings.h"
 #include "probecontroller.h"
 #include "toolpluginmodel.h"
+#include "util.h"
 
-#include "tools/modelinspector/modeltest.h"
+#include <3rdparty/qt/modeltest.h>
 
 #include "remote/server.h"
 #include "remote/remotemodelserver.h"
+#include "remote/serverproxymodel.h"
 #include "remote/selectionmodelserver.h"
 #include "toolpluginerrormodel.h"
 #include "toolfactory.h"
+#include "probeguard.h"
 
 #include <common/objectbroker.h>
 #include <common/streamoperators.h>
@@ -53,14 +61,38 @@
 #include <QDir>
 #include <QLibrary>
 #include <QMouseEvent>
+#include <QUrl>
 #include <QThread>
 #include <QTimer>
 
+#ifdef HAVE_PRIVATE_QT_HEADERS
+#include <private/qobject_p.h>
+#else
+struct QSignalSpyCallbackSet
+{
+  typedef void (*BeginCallback)(QObject *caller, int method_index, void **argv);
+  typedef void (*EndCallback)(QObject *caller, int method_index);
+  BeginCallback signal_begin_callback,
+  slot_begin_callback;
+  EndCallback signal_end_callback,
+  slot_end_callback;
+};
+extern void qt_register_signal_spy_callbacks(const QSignalSpyCallbackSet &callback_set);
+extern QSignalSpyCallbackSet qt_signal_spy_callback_set;
+#endif
+
+#include <algorithm>
 #include <iostream>
 #include <cstdio>
 
 #define IF_DEBUG(x)
 
+#ifdef ENABLE_EXPENSIVE_ASSERTS
+#define EXPENSIVE_ASSERT(x) Q_ASSERT(x)
+#else
+#define EXPENSIVE_ASSERT(x)
+#endif
+
 using namespace GammaRay;
 using namespace std;
 
@@ -68,33 +100,71 @@ QAtomicPointer<Probe> Probe::s_instance = QAtomicPointer<Probe>(0);
 
 namespace GammaRay {
 
-#if (QT_VERSION < QT_VERSION_CHECK(5, 0, 0))
-
-static bool probeConnectCallback(void ** args)
+static void signal_begin_callback(QObject *caller, int method_index, void **argv)
 {
-  QObject *sender = reinterpret_cast<QObject*>(args[0]);
-  const char *signal = reinterpret_cast<const char*>(args[1]);
-  QObject *receiver = reinterpret_cast<QObject*>(args[2]);
-  const char *method = reinterpret_cast<const char*>(args[3]);
-  const Qt::ConnectionType *type = reinterpret_cast<Qt::ConnectionType*>(args[4]);
-  Probe::connectionAdded(sender, signal, receiver, method, *type);
-  return false;
+  if (method_index == 0 || Probe::instance()->filterObject(caller))
+    return;
+
+#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
+  method_index = Util::signalIndexToMethodIndex(caller->metaObject(), method_index);
+#endif
+  Probe::executeSignalCallback([=](const SignalSpyCallbackSet &callbacks) {
+    if (callbacks.signalBeginCallback) {
+      callbacks.signalBeginCallback(caller, method_index, argv);
+    }
+  });
 }
 
-static bool probeDisconnectCallback(void ** args)
+static void signal_end_callback(QObject *caller, int method_index)
 {
-  QObject *sender = reinterpret_cast<QObject*>(args[0]);
-  const char *signal = reinterpret_cast<const char*>(args[1]);
-  QObject *receiver = reinterpret_cast<QObject*>(args[2]);
-  const char *method = reinterpret_cast<const char*>(args[3]);
-  Probe::connectionRemoved(sender, signal, receiver, method);
-  return false;
+  if (method_index == 0)
+    return;
+
+  QMutexLocker locker(Probe::objectLock());
+  if (!Probe::instance()->isValidObject(caller)) // implies filterObject()
+    return; // deleted in the slot
+
+#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
+  method_index = Util::signalIndexToMethodIndex(caller->metaObject(), method_index);
+#endif
+  Probe::executeSignalCallback([=](const SignalSpyCallbackSet &callbacks) {
+    if (callbacks.signalEndCallback) {
+      callbacks.signalEndCallback(caller, method_index);
+    }
+  });
 }
 
-#endif // QT_VERSION
-
-static QItemSelectionModel* selectionModelFactory(QAbstractItemModel* model)
+static void slot_begin_callback(QObject *caller, int method_index, void **argv)
 {
+  if (method_index == 0 || Probe::instance()->filterObject(caller))
+    return;
+
+  Probe::executeSignalCallback([=](const SignalSpyCallbackSet &callbacks) {
+    if (callbacks.slotBeginCallback) {
+            callbacks.slotBeginCallback(caller, method_index, argv);
+    }
+  });
+}
+
+static void slot_end_callback(QObject *caller, int method_index)
+{
+  if (method_index == 0)
+    return;
+
+  QMutexLocker locker(Probe::objectLock());
+  if (!Probe::instance()->isValidObject(caller)) // implies filterObject()
+    return; // deleted in the slot
+
+  Probe::executeSignalCallback([=](const SignalSpyCallbackSet &callbacks) {
+    if (callbacks.slotEndCallback) {
+            callbacks.slotEndCallback(caller, method_index);
+    }
+  });
+}
+
+static QItemSelectionModel *selectionModelFactory(QAbstractItemModel *model)
+{
+  Q_ASSERT(!model->objectName().isEmpty());
   return new SelectionModelServer(model->objectName() + ".selection", model, Probe::instance());
 }
 
@@ -123,14 +193,11 @@ void dumpObject(QObject *obj)
 
 struct Listener
 {
-  Listener()
-    : filterThread(0), trackDestroyed(true)
+  Listener() : trackDestroyed(true)
   {
   }
 
-  QThread *filterThread;
   bool trackDestroyed;
-
   QVector<QObject*> addedBeforeProbeInstance;
 };
 
@@ -138,23 +205,13 @@ Q_GLOBAL_STATIC(Listener, s_listener)
 
 // ensures proper information is returned by isValidObject by
 // locking it in objectAdded/Removed
-class ObjectLock : public QReadWriteLock
-{
-  public:
-    ObjectLock()
-      : QReadWriteLock(QReadWriteLock::Recursive)
-    {
-    }
-};
-Q_GLOBAL_STATIC(ObjectLock, s_lock)
-
+Q_GLOBAL_STATIC_WITH_ARGS(QMutex, s_lock, (QMutex::Recursive))
 
 Probe::Probe(QObject *parent):
   QObject(parent),
   m_objectListModel(new ObjectListModel(this)),
   m_objectTreeModel(new ObjectTreeModel(this)),
   m_metaObjectTreeModel(new MetaObjectTreeModel(this)),
-  m_connectionModel(new ConnectionModel(this)),
   m_toolModel(0),
   m_window(0),
   m_queueTimer(new QTimer(this))
@@ -164,70 +221,73 @@ Probe::Probe(QObject *parent):
 
   ProbeSettings::receiveSettings();
   m_toolModel = new ToolModel(this);
+  auto sortedToolModel = new ServerProxyModel<QSortFilterProxyModel>(this);
+  sortedToolModel->setSourceModel(m_toolModel);
+  sortedToolModel->setDynamicSortFilter(true);
+  sortedToolModel->sort(0);
 
   Server *server = new Server(this);
-  ProbeSettings::sendPort(server->port());
+  ProbeSettings::sendServerAddress(server->externalAddress());
 
   StreamOperators::registerOperators();
   ObjectBroker::setSelectionModelFactoryCallback(selectionModelFactory);
   ObjectBroker::registerObject<ProbeControllerInterface*>(new ProbeController(this));
 
-  registerModel(QLatin1String("com.kdab.GammaRay.ObjectTree"), m_objectTreeModel);
-  registerModel(QLatin1String("com.kdab.GammaRay.ObjectList"), m_objectListModel);
-  registerModel(QLatin1String("com.kdab.GammaRay.MetaObjectModel"), m_metaObjectTreeModel);
-  registerModel(QLatin1String("com.kdab.GammaRay.ToolModel"), m_toolModel);
-  registerModel(QLatin1String("com.kdab.GammaRay.ConnectionModel"), m_connectionModel);
+  registerModel(QStringLiteral("com.kdab.GammaRay.ObjectTree"), m_objectTreeModel);
+  registerModel(QStringLiteral("com.kdab.GammaRay.ObjectList"), m_objectListModel);
+  registerModel(QStringLiteral("com.kdab.GammaRay.MetaObjectModel"), m_metaObjectTreeModel);
+  registerModel(QStringLiteral("com.kdab.GammaRay.ToolModel"), sortedToolModel);
 
-  m_toolSelectionModel = ObjectBroker::selectionModel(m_toolModel);
+  m_toolSelectionModel = ObjectBroker::selectionModel(sortedToolModel);
 
   ToolPluginModel *toolPluginModel = new ToolPluginModel(m_toolModel->plugins(), this);
-  registerModel(QLatin1String("com.kdab.GammaRay.ToolPluginModel"), toolPluginModel);
-  ToolPluginErrorModel *toolPluginErrorModel = new ToolPluginErrorModel(m_toolModel->pluginErrors(), this);
-  registerModel(QLatin1String("com.kdab.GammaRay.ToolPluginErrorModel"), toolPluginErrorModel);
+  registerModel(QStringLiteral("com.kdab.GammaRay.ToolPluginModel"), toolPluginModel);
+  ToolPluginErrorModel *toolPluginErrorModel =
+    new ToolPluginErrorModel(m_toolModel->pluginErrors(), this);
+  registerModel(QStringLiteral("com.kdab.GammaRay.ToolPluginErrorModel"), toolPluginErrorModel);
 
   if (qgetenv("GAMMARAY_MODELTEST") == "1") {
     new ModelTest(m_objectListModel, m_objectListModel);
     new ModelTest(m_objectTreeModel, m_objectTreeModel);
-    new ModelTest(m_connectionModel, m_connectionModel);
     new ModelTest(m_toolModel, m_toolModel);
   }
 
-#if (QT_VERSION < QT_VERSION_CHECK(5, 0, 0))
-  QInternal::registerCallback(QInternal::ConnectCallback, &GammaRay::probeConnectCallback);
-  QInternal::registerCallback(QInternal::DisconnectCallback, &GammaRay::probeDisconnectCallback);
-#endif
-
   m_queueTimer->setSingleShot(true);
   m_queueTimer->setInterval(0);
   connect(m_queueTimer, SIGNAL(timeout()),
-          this, SLOT(queuedObjectsFullyConstructed()));
+          this, SLOT(processQueuedObjectChanges()));
+
+  m_previousSignalSpyCallbackSet.signalBeginCallback = qt_signal_spy_callback_set.signal_begin_callback;
+  m_previousSignalSpyCallbackSet.signalEndCallback =qt_signal_spy_callback_set.signal_end_callback;
+  m_previousSignalSpyCallbackSet.slotBeginCallback = qt_signal_spy_callback_set.slot_begin_callback;
+  m_previousSignalSpyCallbackSet.slotEndCallback = qt_signal_spy_callback_set.slot_end_callback;
+  registerSignalSpyCallbackSet(m_previousSignalSpyCallbackSet); // daisy-chain existing callbacks
 }
 
 Probe::~Probe()
 {
   IF_DEBUG(cerr << "detaching GammaRay probe" << endl;)
 
-#if (QT_VERSION < QT_VERSION_CHECK(5, 0, 0))
-  QInternal::unregisterCallback(QInternal::ConnectCallback, &GammaRay::probeConnectCallback);
-  QInternal::unregisterCallback(QInternal::DisconnectCallback, &GammaRay::probeDisconnectCallback);
-#endif
+  const QSignalSpyCallbackSet prevCallbacks = {
+    m_previousSignalSpyCallbackSet.signalBeginCallback,
+    m_previousSignalSpyCallbackSet.slotBeginCallback,
+    m_previousSignalSpyCallbackSet.signalEndCallback,
+    m_previousSignalSpyCallbackSet.slotEndCallback
+  };
+  qt_register_signal_spy_callbacks(prevCallbacks);
 
   ObjectBroker::clear();
+  ProbeSettings::resetLauncherIdentifier();
 
   s_instance = QAtomicPointer<Probe>(0);
 }
 
-QThread* Probe::filteredThread()
-{
-  return s_listener()->filterThread;
-}
-
 void Probe::setWindow(QObject *window)
 {
   m_window = window;
 }
 
-QObject* Probe::window() const
+QObject *Probe::window() const
 {
   return m_window;
 }
@@ -269,16 +329,18 @@ void Probe::createProbe(bool findExisting)
   // with other QObject's we create and other threads are using. One
   // example are QAbstractSocketEngine.
   IF_DEBUG(cout << "setting up new probe instance" << endl;)
-  s_listener()->filterThread = QThread::currentThread();
-  Probe *probe = new Probe;
-  s_listener()->filterThread = 0;
+  Probe *probe = 0;
+  {
+    ProbeGuard guard;
+    probe = new Probe;
+  }
   IF_DEBUG(cout << "done setting up new probe instance" << endl;)
 
   connect(qApp, SIGNAL(aboutToQuit()), probe, SLOT(deleteLater()));
 
   // now we can get the lock and add items which where added before this point in time
   {
-    QWriteLocker lock(s_lock());
+    QMutexLocker lock(s_lock());
     // now we set the instance while holding the lock,
     // all future calls to object{Added,Removed} will
     // act directly on the data structures there instead
@@ -306,6 +368,11 @@ void Probe::createProbe(bool findExisting)
 
 void Probe::startupHookReceived()
 {
+#ifdef Q_OS_ANDROID
+  QDir root = QDir::home();
+  root.cdUp();
+  Paths::setRootPath(root.absolutePath());
+#endif
   s_listener()->trackDestroyed = false;
 }
 
@@ -328,8 +395,9 @@ void Probe::delayedInit()
   }
   Server::instance()->setLabel(appName);
 
-  if (ProbeSettings::value("InProcessUi", false).toBool())
+  if (ProbeSettings::value(QStringLiteral("InProcessUi"), false).toBool()) {
     showInProcessUi();
+  }
 }
 
 void Probe::showInProcessUi()
@@ -340,25 +408,29 @@ void Probe::showInProcessUi()
   }
 
   IF_DEBUG(cout << "creating GammaRay::MainWindow" << endl;)
-  s_listener()->filterThread = QThread::currentThread();
+  ProbeGuard guard;
 
   QString path = Paths::currentProbePath();
-  if (!path.isEmpty())
+  if (!path.isEmpty()) {
     path += QDir::separator();
-  path += "gammaray_inprocessui";
+  }
+  path += QStringLiteral("gammaray_inprocessui");
   QLibrary lib;
   lib.setFileName(path);
   if (!lib.load()) {
-    std::cerr << "Failed to load in-process UI module: " << qPrintable(lib.errorString()) << std::endl;
+    std::cerr << "Failed to load in-process UI module: "
+              << qPrintable(lib.errorString())
+              << std::endl;
   } else {
-    void(*factory)() = reinterpret_cast<void(*)()>(lib.resolve("gammaray_create_inprocess_mainwindow"));
-    if (!factory)
+    void(*factory)() =
+      reinterpret_cast<void(*)()>(lib.resolve("gammaray_create_inprocess_mainwindow"));
+    if (!factory) {
       std::cerr << Q_FUNC_INFO << ' ' << qPrintable(lib.errorString()) << endl;
-    else
+    } else {
       factory();
+    }
   }
 
-  s_listener()->filterThread = 0;
   IF_DEBUG(cout << "creation done" << endl;)
 }
 
@@ -368,12 +440,33 @@ bool Probe::filterObject(QObject *obj) const
     // shortcut, never filter objects from a different thread
     return false;
   }
-  return obj == this || obj == window() ||
-          Util::descendantOf(this, obj) ||
-          Util::descendantOf(window(), obj);
+
+  QSet<QObject *> visitedObjects;
+  int iteration = 0;
+  QObject *o = obj;
+  do {
+    if (iteration > 100) {
+      // Probably we have a loop in the tree. Do loop detection.
+      if (visitedObjects.contains(o)) {
+        std::cerr << "We detected a loop in the object tree for object " << o;
+        if (!o->objectName().isEmpty()) {
+          std::cerr << " \"" << qPrintable(o->objectName()) << "\"";
+        }
+        std::cerr << " (" << o->metaObject()->className() << ")." << std::endl;
+        return true;
+      }
+      visitedObjects << o;
+    }
+    ++iteration;
+
+    if (o == this || o == window())
+      return true;
+    o = o->parent();
+  } while (o);
+  return false;
 }
 
-void Probe::registerModel(const QString& objectName, QAbstractItemModel* model)
+void Probe::registerModel(const QString &objectName, QAbstractItemModel *model)
 {
   RemoteModelServer *ms = new RemoteModelServer(objectName, model);
   ms->setModel(model);
@@ -395,11 +488,6 @@ QAbstractItemModel *Probe::metaObjectModel() const
   return m_metaObjectTreeModel;
 }
 
-QAbstractItemModel *Probe::connectionModel() const
-{
-  return m_connectionModel;
-}
-
 ToolModel *Probe::toolModel() const
 {
   return m_toolModel;
@@ -416,20 +504,34 @@ bool Probe::isValidObject(QObject *obj) const
   return m_validObjects.contains(obj);
 }
 
-QReadWriteLock *Probe::objectLock()
+QMutex *Probe::objectLock()
 {
   return s_lock();
 }
 
+/*
+ * We need to handle 4 different cases in here:
+ * (1) our thread, from ctor:
+ * - wait until next event-loop re-entry of our thread
+ * - emit objectCreated if object still valid
+ * (2) our thread, after ctor:
+ * - emit objectCreated right away
+ * (3) other thread, from ctor:
+ * - wait until next event-loop re-entry in other thread (FIXME: we do not currently do this!!)
+ * - post information to our thread
+ * - emit objectCreated right away if object still valid
+ * (4) other thread, after ctor:
+ * - post information to our thread
+ * - emit objectCreated there right away if object still valid
+ *
+ * Pre-conditions: lock may or may not be held already, arbitrary thread
+ */
 void Probe::objectAdded(QObject *obj, bool fromCtor)
 {
-  QWriteLocker lock(s_lock());
-  if (s_listener()->filterThread == obj->thread()) {
-    // Ignore
-    IF_DEBUG(cout
-             << "objectAdded Ignore: "
-             << hex << obj
-             << (fromCtor ? " (from ctor)" : "") << endl;)
+  QMutexLocker lock(s_lock());
+
+  // attempt to ignore objects created by GammaRay itself, especially short-lived ones
+  if (fromCtor && ProbeGuard::insideProbe() && obj->thread() == QThread::currentThread()) {
     return;
   }
 
@@ -477,7 +579,7 @@ void Probe::objectAdded(QObject *obj, bool fromCtor)
             Qt::DirectConnection);
   }
 
-  if (!fromCtor && obj->parent() && instance()->m_queuedObjects.contains(obj->parent())) {
+  if (!fromCtor && obj->parent() && instance()->isObjectCreationQueued(obj->parent())) {
     // when a child event triggers a call to objectAdded while inside the ctor
     // the parent is already tracked but it's call to objectFullyConstructed
     // was delayed. hence we must do the same for the child for integrity
@@ -489,43 +591,52 @@ void Probe::objectAdded(QObject *obj, bool fromCtor)
                 << ", p: " << obj->parent() << endl;)
 
   if (fromCtor) {
-    Q_ASSERT(!instance()->m_queuedObjects.contains(obj));
-    instance()->m_queuedObjects << obj;
-    if (!instance()->m_queueTimer->isActive()) {
-      // timers must not be started from a different thread
-      QMetaObject::invokeMethod(instance()->m_queueTimer, "start", Qt::AutoConnection);
-    }
+    instance()->queueCreatedObject(obj);
   } else {
     instance()->objectFullyConstructed(obj);
   }
 }
 
-void Probe::queuedObjectsFullyConstructed()
+// pre-conditions: lock may or may not be held already, our thread
+void Probe::processQueuedObjectChanges()
 {
-  QWriteLocker lock(s_lock());
+  QMutexLocker lock(s_lock());
 
-  IF_DEBUG(cout << Q_FUNC_INFO << " " << m_queuedObjects.size() << endl;)
+  IF_DEBUG(cout << Q_FUNC_INFO << " " << m_queuedObjectChanges.size() << endl;)
 
   // must be called from the main thread via timeout
   Q_ASSERT(QThread::currentThread() == thread());
 
-  // when this is called no object must be in the queue twice
-  // otherwise the cleanup procedures failed
-  Q_ASSERT(m_queuedObjects.size() == m_queuedObjects.toSet().size());
-
-  foreach (QObject *obj, m_queuedObjects) {
-    objectFullyConstructed(obj);
+  foreach (const auto &change, m_queuedObjectChanges) {
+    switch (change.type) {
+      case ObjectChange::Create:
+        objectFullyConstructed(change.obj);
+        break;
+      case ObjectChange::Destroy:
+        emit objectDestroyed(change.obj);
+        break;
+    }
   }
 
   IF_DEBUG(cout << Q_FUNC_INFO << " done" << endl;)
 
-  m_queuedObjects.clear();
+  m_queuedObjectChanges.clear();
+
+  foreach (QObject *obj, m_pendingReparents) {
+    if (!isValidObject(obj))
+      continue;
+    if (filterObject(obj)) // the move might have put it under a hidden parent
+      objectRemoved(obj);
+    else
+      emit objectReparented(obj);
+  }
+  m_pendingReparents.clear();
 }
 
+// pre-condition: lock is held already, our thread
 void Probe::objectFullyConstructed(QObject *obj)
 {
-  // must be write locked
-  Q_ASSERT(!s_lock()->tryLockForRead());
+  Q_ASSERT(thread() == QThread::currentThread());
 
   if (!m_validObjects.contains(obj)) {
     // deleted already
@@ -544,9 +655,12 @@ void Probe::objectFullyConstructed(QObject *obj)
 
   IF_DEBUG(cout << "fully constructed: " << hex << obj << endl;)
 
-  // ensure we know the parent already
-  if (obj->parent() && !m_validObjects.contains(obj->parent())) {
-    objectAdded(obj->parent());
+  // ensure we know all our ancestors already
+  for (QObject *parent = obj->parent(); parent; parent = parent->parent()) {
+    if (!m_validObjects.contains(parent)) {
+      objectAdded(parent); // will also handle any further ancestors
+      break;
+    }
   }
   Q_ASSERT(!obj->parent() || m_validObjects.contains(obj->parent()));
 
@@ -557,7 +671,6 @@ void Probe::objectFullyConstructed(QObject *obj)
     connect(obj, SIGNAL(parentChanged(QQuickItem*)), this, SLOT(objectParentChanged()));
   }
 
-  m_objectListModel->objectAdded(obj);
   m_metaObjectTreeModel->objectAdded(obj);
 
   m_toolModel->objectAdded(obj);
@@ -565,9 +678,18 @@ void Probe::objectFullyConstructed(QObject *obj)
   emit objectCreated(obj);
 }
 
+/*
+ * We have two cases to consider here:
+ * (1) our thread:
+ * - emit objectDestroyed() right away
+ * (2) other thread:
+ * - post information to our thread, emit objectDestroyed() there
+ *
+ * pre-conditions: arbitrary thread, lock may or may not be held already
+ */
 void Probe::objectRemoved(QObject *obj)
 {
-  QWriteLocker lock(s_lock());
+  QMutexLocker lock(s_lock());
 
   if (!isInitialized()) {
     IF_DEBUG(cout
@@ -575,8 +697,9 @@ void Probe::objectRemoved(QObject *obj)
              << hex << obj
              << " have statics: " << s_listener() << endl;)
 
-    if (!s_listener())
+    if (!s_listener()) {
       return;
+    }
 
     QVector<QObject*> &addedBefore = s_listener()->addedBeforeProbeInstance;
     for (QVector<QObject*>::iterator it = addedBefore.begin(); it != addedBefore.end();) {
@@ -594,17 +717,18 @@ void Probe::objectRemoved(QObject *obj)
   bool success = instance()->m_validObjects.remove(obj);
   if (!success) {
     // object was not tracked by the probe, probably a gammaray object
+    EXPENSIVE_ASSERT(!instance()->isObjectCreationQueued(obj));
     return;
   }
 
-  instance()->m_queuedObjects.removeOne(obj);
+  instance()->purgeChangesForObject(obj);
+  EXPENSIVE_ASSERT(!instance()->isObjectCreationQueued(obj));
 
-  instance()->m_objectListModel->objectRemoved(obj);
-
-  instance()->connectionRemoved(obj, 0, 0, 0);
-  instance()->connectionRemoved(0, 0, obj, 0);
-
-  emit instance()->objectDestroyed(obj);
+  if (instance()->thread() == QThread::currentThread()) {
+    emit instance()->objectDestroyed(obj);
+  } else {
+    instance()->queueDestroyedObject(obj);
+  }
 }
 
 void Probe::handleObjectDestroyed(QObject *obj)
@@ -619,44 +743,70 @@ void Probe::objectParentChanged()
   }
 }
 
-void Probe::connectionAdded(QObject *sender, const char *signal, QObject *receiver,
-                            const char *method, Qt::ConnectionType type)
+// pre-condition: we have the lock, arbitrary thread
+void Probe::queueCreatedObject(QObject* obj)
 {
-  if (!isInitialized() || !sender || !receiver ||
-      s_listener()->filterThread == QThread::currentThread())
-  {
-    return;
-  }
+  EXPENSIVE_ASSERT(!isObjectCreationQueued(obj));
 
-  ReadOrWriteLocker lock(s_lock());
-  if (instance()->filterObject(sender) || instance()->filterObject(receiver)) {
-    return;
-  }
-
-  instance()->m_connectionModel->connectionAdded(sender, signal, receiver, method, type);
+  ObjectChange c;
+  c.obj = obj;
+  c.type = ObjectChange::Create;
+  m_queuedObjectChanges.push_back(c);
+  notifyQueuedObjectChanges();
 }
 
-void Probe::connectionRemoved(QObject *sender, const char *signal,
-                              QObject *receiver, const char *method)
+// pre-condition: we have the lock, arbitrary thread
+void Probe::queueDestroyedObject(QObject* obj)
 {
-  if (!isInitialized() || !s_listener() ||
-      s_listener()->filterThread == QThread::currentThread())
-  {
-    return;
-  }
+  ObjectChange c;
+  c.obj = obj;
+  c.type = ObjectChange::Destroy;
+  m_queuedObjectChanges.push_back(c);
+  notifyQueuedObjectChanges();
+}
 
-  ReadOrWriteLocker lock(s_lock());
-  if ((sender && instance()->filterObject(sender)) ||
-      (receiver && instance()->filterObject(receiver))) {
-    return;
-  }
+// pre-condition: we have the lock, arbitrary thread
+bool Probe::isObjectCreationQueued(QObject* obj) const
+{
+  return std::find_if(m_queuedObjectChanges.begin(), m_queuedObjectChanges.end(), [obj](const ObjectChange &c) {
+      return c.obj == obj && c.type == Probe::ObjectChange::Create;
+    }) != m_queuedObjectChanges.end();
+}
 
-  instance()->m_connectionModel->connectionRemoved(sender, signal, receiver, method);
+// pre-condition: we have the lock, arbitrary thread
+void Probe::purgeChangesForObject(QObject* obj)
+{
+  for (int i = 0; i < m_queuedObjectChanges.size(); ++i) {
+    if (m_queuedObjectChanges.at(i).obj == obj && m_queuedObjectChanges.at(i).type == ObjectChange::Create) {
+      m_queuedObjectChanges.remove(i);
+      return;
+    }
+  }
+}
+
+// pre-condition: we have the lock, arbitrary thread
+void Probe::notifyQueuedObjectChanges()
+{
+  if (m_queueTimer->isActive())
+    return;
+
+  if (thread() == QThread::currentThread()) {
+    m_queueTimer->start();
+  } else {
+    static QMetaMethod m;
+    if (m.methodIndex() < 0) {
+      const auto idx = QTimer::staticMetaObject.indexOfMethod("start()");
+      Q_ASSERT(idx >= 0);
+      m = QTimer::staticMetaObject.method(idx);
+      Q_ASSERT(m.methodIndex() >= 0);
+    }
+    m.invoke(m_queueTimer, Qt::QueuedConnection);
+  }
 }
 
 bool Probe::eventFilter(QObject *receiver, QEvent *event)
 {
-  if (s_listener()->filterThread == receiver->thread()) {
+  if (ProbeGuard::insideProbe() && receiver->thread() == QThread::currentThread()) {
     return QObject::eventFilter(receiver, event);
   }
 
@@ -664,7 +814,7 @@ bool Probe::eventFilter(QObject *receiver, QEvent *event)
     QChildEvent *childEvent = static_cast<QChildEvent*>(event);
     QObject *obj = childEvent->child();
 
-    QWriteLocker lock(s_lock());
+    QMutexLocker lock(s_lock());
     const bool tracked = m_validObjects.contains(obj);
     const bool filtered = filterObject(obj);
 
@@ -679,23 +829,42 @@ bool Probe::eventFilter(QObject *receiver, QEvent *event)
         // child added events are sent before qt_addObject is called,
         // so we assumes this comes from the ctor
         objectAdded(obj, true);
-      } else if (!m_queuedObjects.contains(obj)) {
+      } else if (!isObjectCreationQueued(obj) && !isObjectCreationQueued(obj->parent())) {
         // object is known already, just update the position in the tree
         // BUT: only when we did not queue this item before
         IF_DEBUG(cout << "update pos: " << hex << obj << endl;)
+        m_pendingReparents.removeAll(obj);
         emit objectReparented(obj);
       }
     } else if (tracked) {
-      objectRemoved(obj);
+      if (hasReliableObjectTracking()) { // defer processing this until we know its final location
+        m_pendingReparents.push_back(obj);
+        notifyQueuedObjectChanges();
+      } else {
+        objectRemoved(obj);
+      }
+    }
+  }
+
+  // widget only unfortunately, but more precise than ChildAdded/Removed...
+  if (event->type() == QEvent::ParentChange) {
+    QMutexLocker lock(s_lock());
+    const bool tracked = m_validObjects.contains(receiver);
+    const bool filtered = filterObject(receiver);
+    if (!filtered && tracked && !isObjectCreationQueued(receiver) && !isObjectCreationQueued(receiver->parent())) {
+      m_pendingReparents.removeAll(receiver);
+      emit objectReparented(receiver);
     }
   }
 
   // we have no preloading hooks, so recover all objects we see
   if (!hasReliableObjectTracking() && event->type() != QEvent::ChildAdded &&
-      event->type() != QEvent::ChildRemoved && // already handled above
-      event->type() != QEvent::Destroy && event->type() != QEvent::WinIdChange  && // unsafe since emitted from dtors
+      event->type() != QEvent::ChildRemoved &&
+      event->type() != QEvent::ParentChange && // already handled above
+      event->type() != QEvent::Destroy &&
+      event->type() != QEvent::WinIdChange && // unsafe since emitted from dtors
       !filterObject(receiver)) {
-    QWriteLocker lock(s_lock());
+    QMutexLocker lock(s_lock());
     const bool tracked = m_validObjects.contains(receiver);
     if (!tracked) {
       discoverObject(receiver);
@@ -704,8 +873,9 @@ bool Probe::eventFilter(QObject *receiver, QEvent *event)
 
   // filters provided by plugins
   if (!filterObject(receiver)) {
-    foreach (QObject *filter, m_globalEventFilters)
+    foreach (QObject *filter, m_globalEventFilters) {
       filter->eventFilter(receiver, event);
+    }
   }
 
   return QObject::eventFilter(receiver, event);
@@ -716,15 +886,16 @@ void Probe::findExistingObjects()
   discoverObject(QCoreApplication::instance());
 }
 
-void Probe::discoverObject(QObject* obj)
+void Probe::discoverObject(QObject *obj)
 {
   if (!obj) {
     return;
   }
 
-  QWriteLocker lock(s_lock());
-  if (m_validObjects.contains(obj))
+  QMutexLocker lock(s_lock());
+  if (m_validObjects.contains(obj)) {
     return;
+  }
 
   objectAdded(obj);
   foreach (QObject *child, obj->children()) {
@@ -732,7 +903,7 @@ void Probe::discoverObject(QObject* obj)
   }
 }
 
-void Probe::installGlobalEventFilter(QObject* filter)
+void Probe::installGlobalEventFilter(QObject *filter)
 {
   Q_ASSERT(!m_globalEventFilters.contains(filter));
   m_globalEventFilters.push_back(filter);
@@ -743,50 +914,62 @@ bool Probe::hasReliableObjectTracking() const
   return !s_listener()->trackDestroyed;
 }
 
-void Probe::selectObject(QObject* object, const QPoint& pos)
+void Probe::selectObject(QObject *object, const QPoint &pos)
 {
   emit objectSelected(object, pos);
 
-  m_toolSelectionModel->select( m_toolModel->toolForObject(object), QItemSelectionModel::Select | QItemSelectionModel::Clear |
-    QItemSelectionModel::Rows | QItemSelectionModel::Current);
+  const auto srcIdx = m_toolModel->toolForObject(object);
+  const auto proxy = qobject_cast<const QAbstractProxyModel*>(m_toolSelectionModel->model());
+  if (!proxy->sourceModel()) // still detached, ie. no client connected
+    return;
+  const auto idx = proxy->mapFromSource(srcIdx);
+
+  m_toolSelectionModel->select(idx, QItemSelectionModel::Select |
+                               QItemSelectionModel::Clear |
+                               QItemSelectionModel::Rows |
+                               QItemSelectionModel::Current);
 }
 
-void Probe::selectObject(void* object, const QString& typeName)
+void Probe::selectObject(void *object, const QString &typeName)
 {
   emit nonQObjectSelected(object, typeName);
 
-  m_toolSelectionModel->select( m_toolModel->toolForObject(object, typeName), QItemSelectionModel::Select | QItemSelectionModel::Clear |
-    QItemSelectionModel::Rows | QItemSelectionModel::Current);
+  const auto srcIdx = m_toolModel->toolForObject(object, typeName);
+  const auto proxy = qobject_cast<const QAbstractProxyModel*>(m_toolSelectionModel->model());
+  if (!proxy->sourceModel()) // still detached, ie. no client connected
+    return;
+  const auto idx = proxy->mapFromSource(srcIdx);
+
+  m_toolSelectionModel->select(idx, QItemSelectionModel::Select |
+                               QItemSelectionModel::Clear |
+                               QItemSelectionModel::Rows |
+                               QItemSelectionModel::Current);
 }
 
-//BEGIN: SignalSlotsLocationStore
-
-// taken from qobject.cpp
-const int gammaray_flagged_locations_count = 2;
-const char *gammaray_flagged_locations[gammaray_flagged_locations_count] = {0};
-
-static int gammaray_idx = 0;
-
-void SignalSlotsLocationStore::flagLocation(const char *method)
+void Probe::registerSignalSpyCallbackSet(const SignalSpyCallbackSet &callbacks)
 {
-  gammaray_flagged_locations[gammaray_idx] = method;
-  gammaray_idx = (gammaray_idx+1) % gammaray_flagged_locations_count;
+  if (callbacks.isNull())
+    return;
+  m_signalSpyCallbacks.push_back(callbacks);
+  setupSignalSpyCallbacks();
 }
 
-const char *SignalSlotsLocationStore::extractLocation(const char *member)
+void Probe::setupSignalSpyCallbacks()
 {
-  for (int i = 0; i < gammaray_flagged_locations_count; ++i) {
-    if (member == gammaray_flagged_locations[i]) {
-      // signature includes location information after the first null-terminator
-      const char *location = member + qstrlen(member) + 1;
-      if (*location != '\0') {
-        return location;
-      }
-      return 0;
-    }
+  QSignalSpyCallbackSet cbs = { 0, 0, 0, 0 };
+  foreach (const auto &it, m_signalSpyCallbacks) {
+    if (it.signalBeginCallback) cbs.signal_begin_callback = signal_begin_callback;
+    if (it.signalEndCallback) cbs.signal_end_callback = signal_end_callback;
+    if (it.slotBeginCallback) cbs.slot_begin_callback = slot_begin_callback;
+    if (it.slotEndCallback) cbs.slot_end_callback = slot_end_callback;
   }
-  return 0;
+  qt_register_signal_spy_callbacks(cbs);
 }
 
-//END
-
+template <typename Func>
+void Probe::executeSignalCallback(const Func &func)
+{
+  std::for_each(instance()->m_signalSpyCallbacks.constBegin(),
+                instance()->m_signalSpyCallbacks.constEnd(),
+                func);
+}
