@@ -4,9 +4,14 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2010-2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Copyright (C) 2010-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Volker Krause <volker.krause@kdab.com>
 
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  accordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
@@ -41,6 +46,8 @@ class QString;
 
 namespace GammaRay {
 
+struct SignalSpyCallbackSet;
+
 /**
  * @brief An abstract interface for accessing the core GammaRay probe.
  *
@@ -66,12 +73,6 @@ class ProbeInterface
      */
     virtual QAbstractItemModel *objectTreeModel() const = 0;
 
-    /**
-     * Returns the connection model.
-     * @return a pointer to a QAbstractItemModel instance.
-     */
-    virtual QAbstractItemModel *connectionModel() const = 0;
-
     /**
      * Determines if the specified QObject belongs to the GammaRay Probe or Window.
      *
@@ -142,6 +143,15 @@ class ProbeInterface
      * @since 2.1
      */
     virtual void selectObject(void* object, const QString& typeName) = 0;
+
+    /**
+     * Register a signal spy callback set.
+     * Signal indexes provided as arguments are mapped to method indexes, ie. argument semantics
+     * are the same with Qt4 and Qt5.
+     *
+     * @since 2.2
+     */
+    virtual void registerSignalSpyCallbackSet(const SignalSpyCallbackSet &callbacks) = 0;
 };
 
 }
