@@ -4,9 +4,14 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2013-2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Copyright (C) 2013-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Volker Krause <volker.krause@kdab.com>
 
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  accordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
@@ -21,17 +26,18 @@
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include "config-gammaray.h"
+#include <config-gammaray.h>
 #include "probesettings.h"
 
-#include <common/sharedmemorylocker.h>
-#include <common/message.h>
-#include <common/paths.h>
+#include "common/sharedmemorylocker.h"
+#include "common/message.h"
+#include "common/paths.h"
 
 #include <QBuffer>
 #include <QCoreApplication>
 #include <QDebug>
 #include <QDir>
+#include <QUrl>
 #include <QSharedMemory>
 #include <QSystemSemaphore>
 
@@ -69,7 +75,17 @@ void ProbeSettings::receiveSettings()
 #else
     qWarning() << "Unable to receive probe settings, cannot attach to shared memory region" << shm.key() << shm.nativeKey() << ", error is:" << shm.errorString();
 #endif
-    qWarning() << "Continueing anyway, with default settings.";
+    qWarning() << "Continuing anyway, with default settings.";
+
+    // see if we got fallback data via environment variables
+    const QString rootPath = value(QStringLiteral("RootPath")).toString();
+    if (!rootPath.isEmpty())
+      Paths::setRootPath(rootPath);
+    else {
+        const QString probePath = value(QStringLiteral("ProbePath")).toString();
+        if (!probePath.isEmpty())
+            Paths::setRootPath(probePath + QDir::separator() + GAMMARAY_INVERSE_PROBE_DIR);
+    }
     return;
   }
   SharedMemoryLocker locker(&shm);
@@ -87,7 +103,7 @@ void ProbeSettings::receiveSettings()
         msg.payload() >> version;
         if (version != Protocol::version()) {
           qWarning() << "Unable to receive probe settings, mismatching protocol versions (expected:" << Protocol::version() << "got:" << version << ")";
-          qWarning() << "Continueing anyway, but this is likely going to fail.";
+          qWarning() << "Continuing anyway, but this is likely going to fail.";
           return;
         }
         break;
@@ -96,9 +112,14 @@ void ProbeSettings::receiveSettings()
       {
         msg.payload() >> s_probeSettings;
         //qDebug() << Q_FUNC_INFO << s_probeSettings;
-        const QString probePath = value("ProbePath").toString();
-        if (!probePath.isEmpty())
-          Paths::setRootPath(probePath + QDir::separator() + GAMMARAY_INVERSE_PROBE_DIR);
+        const QString rootPath = value(QStringLiteral("RootPath")).toString();
+        if (!rootPath.isEmpty())
+          Paths::setRootPath(rootPath);
+        else {
+            const QString probePath = value(QStringLiteral("ProbePath")).toString();
+            if (!probePath.isEmpty())
+              Paths::setRootPath(probePath + QDir::separator() + GAMMARAY_INVERSE_PROBE_DIR);
+        }
       }
       default:
         continue;
@@ -116,7 +137,14 @@ qint64 ProbeSettings::launcherIdentifier()
   return QCoreApplication::applicationPid();
 }
 
-void ProbeSettings::sendPort(quint16 port)
+void ProbeSettings::resetLauncherIdentifier()
+{
+  // if we were launch by GammaRay, and we later try to re-attach, we need to make sure
+  // to not end up with an outdated launcher id
+  qputenv("GAMMARAY_LAUNCHER_ID", "");
+}
+
+void ProbeSettings::sendServerAddress(const QUrl& addr)
 {
 #ifdef HAVE_SHM
   QSharedMemory shm(QLatin1String("gammaray-") + QString::number(launcherIdentifier()));
@@ -126,7 +154,7 @@ void ProbeSettings::sendPort(quint16 port)
 #else
     qWarning() << "Unable to receive probe settings, cannot attach to shared memory region" << shm.key() << shm.nativeKey() << ", error is:" << shm.errorString();
 #endif
-    qWarning() << "Continueing anyway, with default settings.";
+    qWarning() << "Continuing anyway, with default settings.";
     return;
   }
 
@@ -134,8 +162,8 @@ void ProbeSettings::sendPort(quint16 port)
   QBuffer buffer(&ba);
   buffer.open(QIODevice::WriteOnly);
   {
-    Message msg(Protocol::LauncherAddress, Protocol::ServerPort);
-    msg.payload() << port;
+    Message msg(Protocol::LauncherAddress, Protocol::ServerAddress);
+    msg.payload() << addr;
     msg.write(&buffer);
   }
   buffer.close();
@@ -152,6 +180,6 @@ void ProbeSettings::sendPort(quint16 port)
   QSystemSemaphore sem("gammaray-semaphore-" + QString::number(launcherIdentifier()), QSystemSemaphore::Open);
   sem.release();
 #else
-  Q_UNUSED(port);
+  Q_UNUSED(addr);
 #endif
 }
