@@ -4,9 +4,14 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2013-2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Copyright (C) 2013-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Volker Krause <volker.krause@kdab.com>
 
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  accordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
@@ -36,8 +41,6 @@ class QAbstractItemModel;
 
 namespace GammaRay {
 
-class ConnectionFilterProxyModel;
-
 /** @brief Non-UI part of the property widget. */
 class GAMMARAY_CORE_EXPORT PropertyController : public PropertyControllerInterface
 {
@@ -58,7 +61,9 @@ public:
   template<typename T>
   static void registerExtension()
   {
-    PropertyControllerExtensionFactoryBase *factory = new PropertyControllerExtensionFactory<T>();
+    PropertyControllerExtensionFactoryBase *factory = PropertyControllerExtensionFactory<T>::instance();
+    if (s_extensionFactories.indexOf(factory) >= 0)
+      return;
     s_extensionFactories << factory;
     foreach (PropertyController *instance, s_instances)
       instance->loadExtension(factory);
