@@ -4,9 +4,14 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2011-2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Copyright (C) 2011-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Volker Krause <volker.krause@kdab.com>
 
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  accordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
@@ -38,25 +43,21 @@ namespace GammaRay {
  */
 class ProxyToolFactory : public ProxyFactory<ToolFactory>
 {
+  Q_OBJECT
   public:
     /**
      * @param path Path to the plugin spec file
      */
-    explicit ProxyToolFactory(const QString &path, QObject *parent = 0);
+    explicit ProxyToolFactory(const PluginInfo &pluginInfo, QObject *parent = 0);
 
     /** Returns @c true if the plugin seems valid from all the information we have so far. */
     bool isValid() const;
 
-    virtual QString name() const;
-    virtual QStringList supportedTypes() const;
-    bool isHidden() const;
+    QString name() const Q_DECL_OVERRIDE;
+    QStringList supportedTypes() const Q_DECL_OVERRIDE;
+    bool isHidden() const Q_DECL_OVERRIDE;
 
-    virtual void init(ProbeInterface *probe);
-
-  private:
-    QString m_name;
-    QStringList m_supportedTypes;
-    bool m_hidden;
+    void init(ProbeInterface *probe) Q_DECL_OVERRIDE;
 };
 
 }
