@@ -4,9 +4,14 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2010-2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Copyright (C) 2010-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Volker Krause <volker.krause@kdab.com>
 
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  accordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
@@ -34,7 +39,6 @@
 #define GAMMARAY_TOOLFACTORY_H
 
 #include "probeinterface.h"
-#include <ui/tooluifactory.h>
 
 #include <QMetaType>
 #include <QStringList>
@@ -101,66 +105,27 @@ template <typename Type, typename Tool>
 class StandardToolFactory : public ToolFactory
 {
 public:
-  virtual inline QStringList supportedTypes() const
+  QStringList supportedTypes() const Q_DECL_OVERRIDE
   {
     return QStringList(Type::staticMetaObject.className());
   }
 
-  virtual inline QString id() const
+  QString id() const Q_DECL_OVERRIDE
   {
     return Tool::staticMetaObject.className();
   }
 
-  virtual inline void init(ProbeInterface *probe)
+  void init(ProbeInterface *probe) Q_DECL_OVERRIDE
   {
     new Tool(probe, probe->probe());
   }
 
-  inline bool isHidden() const
+  bool isHidden() const Q_DECL_OVERRIDE
   {
     return false;
   }
 };
 
-/**
- * @brief A templated generic ToolFactory for some data type and Tool.
- */
-// TODO: split this completely, this is just a migration aid
-template <typename Type, typename Tool, typename ToolUi>
-class StandardToolFactory2 : public ToolFactory, public ToolUiFactory
-{
-  public:
-    virtual inline QStringList supportedTypes() const
-    {
-      return QStringList(Type::staticMetaObject.className());
-    }
-
-    virtual inline QString id() const
-    {
-      return Tool::staticMetaObject.className();
-    }
-
-    virtual inline void init(ProbeInterface *probe)
-    {
-      new Tool(probe, probe->probe());
-    }
-
-    inline bool isHidden() const
-    {
-      return false;
-    }
-
-    virtual inline QWidget *createWidget(QWidget *parentWidget)
-    {
-      return new ToolUi(parentWidget);
-    }
-
-    virtual bool remotingSupported() const
-    {
-      return true;
-    }
-};
-
 }
 
 Q_DECLARE_INTERFACE(GammaRay::ToolFactory, "com.kdab.GammaRay.ToolFactory/1.0")
