@@ -2,9 +2,14 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2010-2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Copyright (C) 2010-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Stephen Kelly <stephen.kelly@kdab.com>
 
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  accordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
@@ -25,6 +30,10 @@
 #include "localeaccessormodel.h"
 #include "localedataaccessor.h"
 
+#include <core/remote/serverproxymodel.h>
+
+#include <QSortFilterProxyModel>
+
 using namespace GammaRay;
 
 LocaleInspector::LocaleInspector(ProbeInterface *probe, QObject *parent)
@@ -33,8 +42,16 @@ LocaleInspector::LocaleInspector(ProbeInterface *probe, QObject *parent)
   LocaleDataAccessorRegistry *registry = new LocaleDataAccessorRegistry(this);
 
   LocaleModel *model = new LocaleModel(registry, this);
-  probe->registerModel("com.kdab.GammaRay.LocaleModel", model);
+  auto proxy = new ServerProxyModel<QSortFilterProxyModel>(this);
+  proxy->setSourceModel(model);
+  probe->registerModel(QStringLiteral("com.kdab.GammaRay.LocaleModel"), proxy);
+
+
   LocaleAccessorModel *accessorModel = new LocaleAccessorModel(registry, this);
-  probe->registerModel("com.kdab.GammaRay.LocaleAccessorModel", accessorModel);
+  probe->registerModel(QStringLiteral("com.kdab.GammaRay.LocaleAccessorModel"), accessorModel);
 }
 
+QString LocaleInspectorFactory::name() const
+{
+  return tr("Locales");
+}
