@@ -4,9 +4,14 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2010-2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Copyright (C) 2010-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Milian Wolff <milian.wolff@kdab.com>
 
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  accordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
@@ -26,34 +31,60 @@
 
 #include "backtrace.h"
 
+#include <core/probeguard.h>
+#include <core/remote/serverproxymodel.h>
+
 #include "common/objectbroker.h"
 #include "common/endpoint.h"
 
 #include <QCoreApplication>
 #include <QDebug>
 #include <QMutex>
+#include <QSortFilterProxyModel>
 #include <QThread>
 
-static QTextStream cerr(stdout);
+#include <iostream>
 
 using namespace GammaRay;
 
+#if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
+typedef QtMsgHandler MessageHandlerCallback;
+static MessageHandlerCallback (*const installMessageHandler)(MessageHandlerCallback) = qInstallMsgHandler;
+#else
+typedef QtMessageHandler MessageHandlerCallback;
+static MessageHandlerCallback (*const installMessageHandler)(MessageHandlerCallback) = qInstallMessageHandler;
+#endif
+
 static MessageModel *s_model = 0;
-static QtMsgHandler s_handler = 0;
+static MessageHandlerCallback s_handler = 0;
 static bool s_handlerDisabled = false;
 static QMutex s_mutex(QMutex::Recursive);
 
-void handleMessage(QtMsgType type, const char *msg)
+#if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
+static void handleMessage(QtMsgType type, const char *rawMsg)
+#else
+static void handleMessage(QtMsgType type, const QMessageLogContext &context, const QString &msg)
+#endif
 {
   ///WARNING: do not trigger *any* kind of debug output here
   ///         this would trigger an infinite loop and hence crash!
 
+#if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
+  const QString msg = QString::fromLocal8Bit(rawMsg);
+#endif
+
   DebugMessage message;
   message.type = type;
-  message.message = QString::fromLocal8Bit(msg);
+  message.message = msg;
   message.time = QTime::currentTime();
+#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
+  message.category = context.category;
+  message.file = context.file;
+  message.function = context.function;
+  message.line = context.line;
+#endif
 
-  if (type == QtCriticalMsg || type == QtFatalMsg || type == QtWarningMsg) {
+  if (type == QtCriticalMsg || type == QtFatalMsg || (type == QtWarningMsg && !ProbeGuard::insideProbe())) {
     message.backtrace = getBacktrace(50);
     // remove trailing internal functions
     // be a bit careful and first make sure that we find this function...
@@ -72,14 +103,14 @@ void handleMessage(QtMsgType type, const char *msg)
 
   if (!message.backtrace.isEmpty() && (qgetenv("GAMMARAY_UNITTEST") == "1" || type == QtFatalMsg)) {
     if (type == QtFatalMsg) {
-      cerr << "QFatal in " << qPrintable(qApp->applicationName()) << " (" << qPrintable(qApp->applicationFilePath()) << ')' << endl;
+      std::cerr << "QFatal in " << qPrintable(qApp->applicationName()) << " (" << qPrintable(qApp->applicationFilePath()) << ')' << std::endl;
     }
-    cerr << "START BACKTRACE:" << endl;
+    std::cerr << "START BACKTRACE:" << std::endl;
     int i = 0;
     foreach (const QString &frame, message.backtrace) {
-      cerr << (++i) << "\t" << frame << endl;
+      std::cerr << (++i) << "\t" << qPrintable(frame) << std::endl;
     }
-    cerr << "END BACKTRACE" << endl;
+    std::cerr << "END BACKTRACE" << std::endl;
   }
 
   if (type == QtFatalMsg && qgetenv("GAMMARAY_GDB") != "1" && qgetenv("GAMMARAY_UNITTEST") != "1") {
@@ -94,14 +125,21 @@ void handleMessage(QtMsgType type, const char *msg)
   // handler during that time
   QMutexLocker lock(&s_mutex);
   s_handlerDisabled = true;
-  qInstallMsgHandler(s_handler);
-#if (QT_VERSION >= QT_VERSION_CHECK(5,0,0))
-  static QMessageLogContext context;
-  qt_message_output(type, context, msg);
+  if (s_handler) { // try a direct call to the previous handler first, that avoids triggering the recursion detection in Qt5
+#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
+    s_handler(type, context, msg);
 #else
-  qt_message_output(type, msg);
+    s_handler(type, rawMsg);
 #endif
-  qInstallMsgHandler(handleMessage);
+  } else {
+    installMessageHandler(s_handler);
+#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
+    qt_message_output(type, context, msg);
+#else
+    qt_message_output(type, rawMsg);
+#endif
+    installMessageHandler(handleMessage);
+  }
   s_handlerDisabled = false;
   lock.unlock();
 
@@ -120,7 +158,13 @@ MessageHandler::MessageHandler(ProbeInterface *probe, QObject *parent)
   Q_ASSERT(s_model == 0);
   s_model = m_messageModel;
 
-  probe->registerModel("com.kdab.GammaRay.MessageModel", m_messageModel);
+  auto proxy = new ServerProxyModel<QSortFilterProxyModel>(this);
+  proxy->addRole(MessageModelRole::Type);
+  proxy->addRole(MessageModelRole::Line);
+  proxy->addRole(MessageModelRole::Backtrace);
+  proxy->setSourceModel(m_messageModel);
+  proxy->setSortRole(MessageModelRole::Sort);
+  probe->registerModel(QStringLiteral("com.kdab.GammaRay.MessageModel"), proxy);
 
   // install handler directly, catches most cases,
   // i.e. user has no special handler or the handler
@@ -136,10 +180,10 @@ MessageHandler::~MessageHandler()
   QMutexLocker lock(&s_mutex);
 
   s_model = 0;
-  QtMsgHandler oldHandler = qInstallMsgHandler(s_handler);
+  MessageHandlerCallback oldHandler = installMessageHandler(s_handler);
   if (oldHandler != handleMessage) {
     // ups, the app installed it's own handler after ours...
-    qInstallMsgHandler(oldHandler);
+    installMessageHandler(oldHandler);
   }
   s_handler = 0;
 }
@@ -152,7 +196,7 @@ void MessageHandler::ensureHandlerInstalled()
     return;
   }
 
-  QtMsgHandler prevHandler = qInstallMsgHandler(handleMessage);
+  MessageHandlerCallback prevHandler = installMessageHandler(handleMessage);
 
   if (prevHandler != handleMessage) {
     s_handler = prevHandler;
@@ -174,3 +218,7 @@ MessageHandlerFactory::MessageHandlerFactory(QObject* parent): QObject(parent)
 {
 }
 
+QString MessageHandlerFactory::name() const
+{
+  return tr("Messages");
+}
