@@ -4,9 +4,14 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2010-2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Copyright (C) 2010-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Volker Krause <volker.krause@kdab.com>
 
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  accordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
@@ -24,7 +29,10 @@
 #include "metatypebrowser.h"
 #include "metatypesmodel.h"
 
-#include "common/objectbroker.h"
+#include <common/objectbroker.h>
+#include <core/remote/serverproxymodel.h>
+
+#include <QSortFilterProxyModel>
 
 using namespace GammaRay;
 
@@ -32,6 +40,12 @@ MetaTypeBrowser::MetaTypeBrowser(ProbeInterface *probe, QObject *parent)
   : QObject(parent)
 {
   MetaTypesModel *mtm = new MetaTypesModel(this);
-  probe->registerModel("com.kdab.GammaRay.MetaTypeModel", mtm);
+  auto proxy = new ServerProxyModel<QSortFilterProxyModel>(this);
+  proxy->setSourceModel(mtm);
+  probe->registerModel(QStringLiteral("com.kdab.GammaRay.MetaTypeModel"), proxy);
 }
 
+QString MetaTypeBrowserFactory::name() const
+{
+  return tr("Meta Types");
+}
