@@ -4,9 +4,14 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2012-2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Copyright (C) 2012-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Volker Krause <volker.krause@kdab.com>
 
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  accordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
@@ -36,9 +41,6 @@ class MimeTypes : public QObject
   public:
     explicit MimeTypes(ProbeInterface *probe, QObject *parent = 0);
     ~MimeTypes();
-
-  private:
-    QStandardItemModel *m_model;
 };
 
 class MimeTypesFactory : public QObject, public StandardToolFactory<QObject, MimeTypes>
@@ -50,10 +52,7 @@ class MimeTypesFactory : public QObject, public StandardToolFactory<QObject, Mim
     {
     }
 
-    virtual inline QString name() const
-    {
-      return tr("Mime Types");
-    }
+    QString name() const Q_DECL_OVERRIDE;
 };
 
 }
