@@ -4,9 +4,14 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2010-2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Copyright (C) 2010-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Volker Krause <volker.krause@kdab.com>
 
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  accordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
@@ -28,15 +33,16 @@
 
 #include "common/modelinspectorinterface.h"
 
+class QAbstractItemModel;
 class QItemSelection;
 class QItemSelectionModel;
 
 namespace GammaRay {
 
-class ModelModel;
 class ModelCellModel;
 class ModelTester;
 class RemoteModelServer;
+class SafetyFilterProxyModel;
 
 class ModelInspector : public ModelInspectorInterface
 {
@@ -50,13 +56,16 @@ class ModelInspector : public ModelInspectorInterface
     void selectionChanged(const QItemSelection &selected);
 
     void objectSelected(QObject* object);
+    void objectCreated(QObject *object);
 
   private:
-    ModelModel *m_modelModel;
+    ProbeInterface *m_probe;
+    QAbstractItemModel *m_modelModel;
     QItemSelectionModel *m_modelSelectionModel;
 
     RemoteModelServer *m_modelContentServer;
     QItemSelectionModel *m_modelContentSelectionModel;
+    SafetyFilterProxyModel *m_safetyFilterProxyModel;
 
     ModelCellModel *m_cellModel;
 
@@ -72,7 +81,7 @@ class ModelInspectorFactory : public QObject, public StandardToolFactory<QAbstra
     {
     }
 
-    QString name() const;
+    QString name() const Q_DECL_OVERRIDE;
 };
 
 }
