@@ -4,9 +4,14 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2010-2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Copyright (C) 2010-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Volker Krause <volker.krause@kdab.com>
 
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  accordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
@@ -33,6 +38,8 @@
 #include <common/objectbroker.h>
 #include <common/objectmodel.h>
 
+#include <3rdparty/kde/krecursivefilterproxymodel.h>
+
 #include <QCoreApplication>
 #include <QItemSelectionModel>
 
@@ -43,9 +50,13 @@ ObjectInspector::ObjectInspector(ProbeInterface *probe, QObject *parent)
 {
   registerPCExtensions();
 
-  m_propertyController = new PropertyController("com.kdab.GammaRay.ObjectInspector", this);
+  m_propertyController = new PropertyController(QStringLiteral("com.kdab.GammaRay.ObjectInspector"), this);
 
-  m_selectionModel = ObjectBroker::selectionModel(ObjectBroker::model("com.kdab.GammaRay.ObjectTree"));
+  auto proxy = new KRecursiveFilterProxyModel(this);
+  proxy->setSourceModel(probe->objectTreeModel());
+  probe->registerModel(QStringLiteral("com.kdab.GammaRay.ObjectInspectorTree"), proxy);
+
+  m_selectionModel = ObjectBroker::selectionModel(proxy);
 
   connect(m_selectionModel,
           SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
@@ -118,3 +129,8 @@ void ObjectInspector::registerPCExtensions()
   PropertyController::registerExtension<PropertiesExtension>();
   PropertyController::registerExtension<ConnectionsExtension>();
 }
+
+QString ObjectInspectorFactory::name() const
+{
+  return tr("Objects");
+}
