@@ -4,9 +4,14 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2010-2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Copyright (C) 2010-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Stephen Kelly <stephen.kelly@kdab.com>
 
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  accordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
@@ -27,6 +32,8 @@
 #include "qt/resourcemodel.h"
 #include "common/objectbroker.h"
 
+#include <core/remote/serverproxymodel.h>
+
 #include <QDebug>
 #include <QItemSelectionModel>
 #include <QPixmap>
@@ -37,20 +44,39 @@ ResourceBrowser::ResourceBrowser(ProbeInterface *probe, QObject *parent)
   : ResourceBrowserInterface(parent)
 {
   ResourceModel *resourceModel = new ResourceModel(this);
-  ResourceFilterModel *proxy = new ResourceFilterModel(this);
+  auto proxy = new ServerProxyModel<ResourceFilterModel>(this);
   proxy->setSourceModel(resourceModel);
-  probe->registerModel("com.kdab.GammaRay.ResourceModel", proxy);
+  probe->registerModel(QStringLiteral("com.kdab.GammaRay.ResourceModel"), proxy);
   QItemSelectionModel *selectionModel = ObjectBroker::selectionModel(proxy);
   connect(selectionModel, SIGNAL(currentChanged(QModelIndex,QModelIndex)),
           this, SLOT(currentChanged(QModelIndex)));
 }
 
+void ResourceBrowser::downloadResource(const QString &sourceFilePath, const QString &targetFilePath)
+{
+  const QFileInfo fi(sourceFilePath);
+
+  if (fi.isFile()) {
+    static const QStringList l = QStringList() << QStringLiteral("jpg") << QStringLiteral("png") << QStringLiteral("jpeg");
+    if (l.contains(fi.suffix())) {
+      emit resourceDownloaded(targetFilePath, QPixmap(fi.absoluteFilePath()));
+    } else {
+      QFile f(fi.absoluteFilePath());
+      if (f.open(QFile::ReadOnly | QFile::Text)) {
+        emit resourceDownloaded(targetFilePath, f.readAll());
+      } else {
+        qWarning() << "Failed to open" << fi.absoluteFilePath();
+      }
+    }
+  }
+}
+
 void ResourceBrowser::currentChanged(const QModelIndex &current)
 {
   const QFileInfo fi(current.data(ResourceModel::FilePathRole).toString());
 
   if (fi.isFile()) {
-    static const QStringList l = QStringList() << "jpg" << "png" << "jpeg";
+    static const QStringList l = QStringList() << QStringLiteral("jpg") << QStringLiteral("png") << QStringLiteral("jpeg");
     if (l.contains(fi.suffix())) {
       emit resourceSelected(QPixmap(fi.absoluteFilePath()));
     } else {
@@ -67,3 +93,7 @@ void ResourceBrowser::currentChanged(const QModelIndex &current)
   }
 }
 
+QString ResourceBrowserFactory::name() const
+{
+  return tr("Resources");
+}
