@@ -4,9 +4,14 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2010-2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Copyright (C) 2010-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Volker Krause <volker.krause@kdab.com>
 
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  accordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
@@ -42,27 +47,32 @@ TextDocumentInspector::TextDocumentInspector(ProbeInterface *probe, QObject *par
   ObjectTypeFilterProxyModel<QTextDocument> *documentFilter =
     new ObjectTypeFilterProxyModel<QTextDocument>(this);
   documentFilter->setSourceModel(probe->objectListModel());
-  probe->registerModel("com.kdab.GammaRay.TextDocumentsModel", documentFilter);
+  probe->registerModel(QStringLiteral("com.kdab.GammaRay.TextDocumentsModel"), documentFilter);
 
   QItemSelectionModel *selectionModel = ObjectBroker::selectionModel(documentFilter);
   connect(selectionModel, SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
           SLOT(documentSelected(QItemSelection,QItemSelection)));
 
   m_textDocumentModel = new TextDocumentModel(this);
-  probe->registerModel("com.kdab.GammaRay.TextDocumentModel", m_textDocumentModel);
+  probe->registerModel(QStringLiteral("com.kdab.GammaRay.TextDocumentModel"), m_textDocumentModel);
 
   selectionModel = ObjectBroker::selectionModel(m_textDocumentModel);
   connect(selectionModel, SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
           SLOT(documentElementSelected(QItemSelection,QItemSelection)));
 
   m_textDocumentFormatModel = new TextDocumentFormatModel(this);
-  probe->registerModel("com.kdab.GammaRay.TextDocumentFormatModel", m_textDocumentFormatModel);
+  probe->registerModel(QStringLiteral("com.kdab.GammaRay.TextDocumentFormatModel"), m_textDocumentFormatModel);
 }
 
 void TextDocumentInspector::documentSelected(const QItemSelection &selected,
                                              const QItemSelection &deselected)
 {
   Q_UNUSED(deselected);
+  if (selected.isEmpty()) {
+    m_textDocumentModel->setDocument(0);
+    return;
+  }
+
   const QModelIndex selectedRow = selected.first().topLeft();
   QObject *selectedObj = selectedRow.data(ObjectModel::ObjectRole).value<QObject*>();
   QTextDocument *doc = qobject_cast<QTextDocument*>(selectedObj);
@@ -82,3 +92,7 @@ void TextDocumentInspector::documentElementSelected(const QItemSelection &select
   m_textDocumentFormatModel->setFormat(f);
 }
 
+QString TextDocumentInspectorFactory::name() const
+{
+  return tr("Text Documents");
+}
