@@ -4,9 +4,14 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2013-2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Copyright (C) 2013-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Volker Krause <volker.krause@kdab.com>
 
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  accordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
@@ -35,6 +40,7 @@
 #include <QDebug>
 #include <QDir>
 #include <QIcon>
+#include <QMatrix4x4>
 #include <QMetaEnum>
 #include <QMetaObject>
 #include <QObject>
@@ -45,6 +51,9 @@
 #include <QSize>
 #include <QStringList>
 #include <QTextFormat>
+#include <QVector2D>
+#include <QVector3D>
+#include <QVector4D>
 
 using namespace GammaRay;
 
@@ -54,8 +63,62 @@ namespace GammaRay
 struct VariantHandlerRepository
 {
   QHash<int, VariantHandler::Converter<QString>*> stringConverters;
+  QVector<VariantHandler::GenericStringConverter> genericStringConverters;
 };
 
+static QString displayMatrix4x4(const QMatrix4x4 &matrix)
+{
+  QStringList rows;
+  rows.reserve(4);
+  for (int i = 0; i < 4; ++i) {
+    QStringList cols;
+    cols.reserve(4);
+    for (int j = 0; j < 4; ++j) {
+      cols.push_back(QString::number(matrix(i, j)));
+    }
+    rows.push_back(cols.join(QStringLiteral(" ")));
+  }
+  return '[' + rows.join(QStringLiteral(", ")) + ']';
+}
+
+static QString displayMatrix4x4(const QMatrix4x4 *matrix)
+{
+  if (matrix) {
+    return displayMatrix4x4(*matrix);
+  }
+  return QStringLiteral("<null>");
+}
+
+template <int Dim, typename T>
+static QString displayVector(const T &vector)
+{
+  QStringList v;
+  for (int i = 0; i < Dim; ++i)
+    v.push_back(QString::number(vector[i]));
+  return '[' + v.join(QStringLiteral(", ")) + ']';
+}
+
+#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
+static QString displayShaderType(const QOpenGLShader::ShaderType type)
+{
+  QStringList types;
+#define ST(t) if (type & QOpenGLShader::t) types.push_back(QStringLiteral(#t));
+  ST(Vertex)
+  ST(Fragment)
+#if QT_VERSION >= QT_VERSION_CHECK(5, 1, 0)
+  ST(Geometry)
+  ST(TessellationControl)
+  ST(TessellationEvaluation)
+  ST(Compute)
+#endif
+#undef ST
+
+  if (types.isEmpty())
+    return QStringLiteral("<none>");
+  return types.join(QStringLiteral(" | "));
+}
+#endif
+
 }
 
 Q_GLOBAL_STATIC(VariantHandlerRepository, s_variantHandlerRepository)
@@ -76,42 +139,44 @@ QString VariantHandler::displayString(const QVariant &value)
     if (icon.isNull()) {
       return QObject::tr("<no icon>");
     }
+    const auto sizes = icon.availableSizes();
     QStringList l;
-    foreach (const QSize &size, icon.availableSizes()) {
+    l.reserve(sizes.size());
+    foreach (const QSize &size, sizes) {
       l.push_back(displayString(size));
     }
-    return l.join(QLatin1String(", "));
+    return l.join(QStringLiteral(", "));
   }
   case QVariant::Line:
     return
-      QString::fromUtf8("%1 x %2 → %3 x %4").
+      QStringLiteral("%1, %2 → %3, %4").
         arg(value.toLine().x1()).arg(value.toLine().y1()).
         arg(value.toLine().x2()).arg(value.toLine().y2());
 
   case QVariant::LineF:
     return
-      QString::fromUtf8("%1 x %2 → %3 x %4").
+      QStringLiteral("%1, %2 → %3, %4").
         arg(value.toLineF().x1()).arg(value.toLineF().y1()).
         arg(value.toLineF().x2()).arg(value.toLineF().y2());
 
   case QVariant::Locale:
-    return value.value<QLocale>().name();
+    return value.toLocale().name();
 
   case QVariant::Point:
     return
-      QString::fromLatin1("%1x%2").
+      QStringLiteral("%1, %2").
         arg(value.toPoint().x()).
         arg(value.toPoint().y());
 
   case QVariant::PointF:
     return
-      QString::fromLatin1("%1x%2").
+      QStringLiteral("%1, %2").
         arg(value.toPointF().x()).
         arg(value.toPointF().y());
 
   case QVariant::Rect:
     return
-      QString::fromLatin1("%1x%2 %3x%4").
+      QStringLiteral("%1, %2 %3 x %4").
         arg(value.toRect().x()).
         arg(value.toRect().y()).
         arg(value.toRect().width()).
@@ -119,7 +184,7 @@ QString VariantHandler::displayString(const QVariant &value)
 
   case QVariant::RectF:
     return
-      QString::fromLatin1("%1x%2 %3x%4").
+      QStringLiteral("%1, %2 %3 x %4").
         arg(value.toRectF().x()).
         arg(value.toRectF().y()).
         arg(value.toRectF().width()).
@@ -129,12 +194,12 @@ QString VariantHandler::displayString(const QVariant &value)
   {
     const QRegion region = value.value<QRegion>();
     if (region.isEmpty()) {
-      return QLatin1String("<empty>");
+      return QStringLiteral("<empty>");
     }
     if (region.rectCount() == 1) {
-      return displayString(region.rects().first());
+      return displayString(region.rects().at(0));
     } else {
-      return QString::fromLatin1("<%1 rects>").arg(region.rectCount());
+      return QStringLiteral("<%1 rects>").arg(region.rectCount());
     }
   }
 
@@ -142,31 +207,42 @@ QString VariantHandler::displayString(const QVariant &value)
   {
     const QPalette pal = value.value<QPalette>();
     if (pal == qApp->palette()) {
-      return QLatin1String("<inherited>");
+      return QStringLiteral("<inherited>");
     }
-    return QLatin1String("<custom>");
+    return QStringLiteral("<custom>");
   }
 
   case QVariant::Size:
     return
-      QString::fromLatin1("%1x%2").
+      QStringLiteral("%1 x %2").
         arg(value.toSize().width()).
         arg(value.toSize().height());
 
   case QVariant::SizeF:
     return
-      QString::fromLatin1("%1x%2").
+      QStringLiteral("%1 x %2").
         arg(value.toSizeF().width()).
         arg(value.toSizeF().height());
 
   case QVariant::StringList:
-    return value.toStringList().join(", ");
+  {
+    const auto l = value.toStringList();
+#if QT_VERSION >= QT_VERSION_CHECK(5, 2, 0)
+    if (l.isEmpty())
+      return QStringLiteral("<empty>");
+    if (l.size() == 1)
+      return l.at(0);
+    return QStringLiteral("<%1 entries>").arg(l.size());
+#else
+    return l.join(", ");
+#endif
+  }
 
   case QVariant::Transform:
   {
     const QTransform t = value.value<QTransform>();
     return
-      QString::fromLatin1("[%1 %2 %3, %4 %5 %6, %7 %8 %9]").
+      QStringLiteral("[%1 %2 %3, %4 %5 %6, %7 %8 %9]").
         arg(t.m11()).arg(t.m12()).arg(t.m13()).
         arg(t.m21()).arg(t.m22()).arg(t.m23()).
         arg(t.m31()).arg(t.m32()).arg(t.m33());
@@ -190,7 +266,7 @@ QString VariantHandler::displayString(const QVariant &value)
       typeStr = QObject::tr("percentage");
       break;
     }
-    return QString::fromLatin1("%1 (%2)").arg(l.rawValue()).arg(typeStr);
+    return QStringLiteral("%1 (%2)").arg(l.rawValue()).arg(typeStr);
   }
 
   if (value.userType() == qMetaTypeId<QPainterPath>()) {
@@ -212,14 +288,32 @@ QString VariantHandler::displayString(const QVariant &value)
     return Util::displayString(value.value<QObject*>());
   }
 
+  if (value.userType() == qMetaTypeId<QMatrix4x4>()) {
+    return displayMatrix4x4(value.value<QMatrix4x4>());
+  }
+
+  if (value.userType() == qMetaTypeId<const QMatrix4x4*>()) {
+    return displayMatrix4x4(value.value<const QMatrix4x4*>());
+  }
+
+#if QT_VERSION >= QT_VERSION_CHECK(5, 2, 0)
+  if (value.userType() == qMetaTypeId<QVector2D>())
+    return displayVector<2>(value.value<QVector2D>());
+  if (value.userType() == qMetaTypeId<QVector3D>())
+    return displayVector<3>(value.value<QVector3D>());
+  if (value.userType() == qMetaTypeId<QVector4D>())
+    return displayVector<4>(value.value<QVector4D>());
+#endif
+
 #if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
   if (value.userType() == qMetaTypeId<QSet<QByteArray> >()) {
     const QSet<QByteArray> set = value.value<QSet<QByteArray> >();
     QStringList l;
+    l.reserve(set.size());
     foreach (const QByteArray &b, set) {
       l.push_back(QString::fromUtf8(b));
     }
-    return l.join(", ");
+    return l.join(QStringLiteral(", "));
   }
 
   if (value.userType() == qMetaTypeId<QSurfaceFormat>()) {
@@ -227,16 +321,16 @@ QString VariantHandler::displayString(const QVariant &value)
     QString s;
     switch (format.renderableType()) {
     case QSurfaceFormat::DefaultRenderableType:
-      s += "Default";
+      s += QStringLiteral("Default");
       break;
     case QSurfaceFormat::OpenGL:
-      s += "OpenGL";
+      s += QStringLiteral("OpenGL");
       break;
     case QSurfaceFormat::OpenGLES:
-      s += "OpenGL ES";
+      s += QStringLiteral("OpenGL ES");
       break;
     case QSurfaceFormat::OpenVG:
-      s += "OpenVG";
+      s += QStringLiteral("OpenVG");
       break;
     }
 
@@ -244,10 +338,10 @@ QString VariantHandler::displayString(const QVariant &value)
          '.' + QString::number(format.minorVersion());
     switch (format.profile()) {
     case QSurfaceFormat::CoreProfile:
-      s += " core";
+      s += QStringLiteral(" core");
       break;
     case QSurfaceFormat::CompatibilityProfile:
-      s += " compat";
+      s += QStringLiteral(" compat");
       break;
     case QSurfaceFormat::NoProfile:
       break;
@@ -262,22 +356,22 @@ QString VariantHandler::displayString(const QVariant &value)
     s += " Depth: " + QString::number(format.depthBufferSize());
     s += " Stencil: " + QString::number(format.stencilBufferSize());
 
-    s += " Buffer: ";
+    s += QStringLiteral(" Buffer: ");
     switch (format.swapBehavior()) {
     case QSurfaceFormat::DefaultSwapBehavior:
-      s += "default";
+      s += QStringLiteral("default");
       break;
     case QSurfaceFormat::SingleBuffer:
-      s += "single";
+      s += QStringLiteral("single");
       break;
     case QSurfaceFormat::DoubleBuffer:
-      s += "double";
+      s += QStringLiteral("double");
       break;
     case QSurfaceFormat::TripleBuffer:
-      s += "triple";
+      s += QStringLiteral("triple");
       break;
     default:
-      s += "unknown";
+      s += QStringLiteral("unknown");
     }
 
     return s;
@@ -303,7 +397,10 @@ QString VariantHandler::displayString(const QVariant &value)
     }
   }
 
-#endif
+  if (value.userType() == qMetaTypeId<QOpenGLShader::ShaderType>())
+    return displayShaderType(value.value<QOpenGLShader::ShaderType>());
+
+#endif // Qt5
 
   // enums
   const QString enumStr = Util::enumToString(value);
@@ -318,26 +415,36 @@ QString VariantHandler::displayString(const QVariant &value)
     return (*it.value())(value);
   }
 
-  // recurse into sequences
 #if QT_VERSION >= QT_VERSION_CHECK(5, 2, 0)
   if (value.canConvert<QVariantList>()) {
-    QStringList s;
     QSequentialIterable it = value.value<QSequentialIterable>();
-    int emptyStrings = 0;
-    foreach (const QVariant &v, it) {
-      s.push_back(displayString(v));
-      if (s.last().isEmpty())
-        ++emptyStrings;
+    if (it.size() == 0) {
+      return QStringLiteral("<empty>");
+    } else {
+      return QStringLiteral("<%1 entries>").arg(it.size());
+    }
+  }
+  if (value.canConvert<QVariantHash>()) {
+    auto it = value.value<QAssociativeIterable>();
+    if (it.size() == 0) {
+      return QStringLiteral("<empty>");
+    } else {
+      return QStringLiteral("<%1 entries>").arg(it.size());
     }
-    if (it.size() == 0)
-      return QObject::tr("<empty>");
-    else if (it.size() == emptyStrings) // we don't know the content either
-      return QObject::tr("%1 entries").arg(emptyStrings);
-    else
-      return s.join(", ");
   }
 #endif
 
+  // generic converters
+  QVector<VariantHandler::GenericStringConverter> genStrConverters =
+    s_variantHandlerRepository()->genericStringConverters;
+  foreach (const GenericStringConverter &converter, genStrConverters) {
+    bool ok = false;
+    const QString s = converter(value, &ok);
+    if (ok) {
+      return s;
+    }
+  }
+
   return value.toString();
 }
 
@@ -363,6 +470,7 @@ QVariant VariantHandler::decoration(const QVariant &value)
     if (c.isValid()) {
       QPixmap p(16, 16);
       QPainter painter(&p);
+      Util::drawTransparencyPattern(&painter, p.rect(), 4);
       painter.setBrush(QBrush(c));
       painter.drawRect(0, 0, p.width() - 1, p.height() - 1);
       return p;
@@ -415,3 +523,21 @@ void VariantHandler::registerStringConverter(int type, Converter<QString> *conve
 {
   s_variantHandlerRepository()->stringConverters.insert(type, converter);
 }
+
+void VariantHandler::registerGenericStringConverter(
+  VariantHandler::GenericStringConverter converter)
+{
+  s_variantHandlerRepository()->genericStringConverters.push_back(converter);
+}
+
+QVariant VariantHandler::serializableVariant(const QVariant& value)
+{
+  if (value.userType() == qMetaTypeId<const QMatrix4x4*>()) {
+    const QMatrix4x4 *m = value.value<const QMatrix4x4*>();
+    if (!m)
+      return QVariant();
+    return QVariant::fromValue(QMatrix4x4(*m));
+  }
+
+  return value;
+}
