@@ -4,9 +4,14 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Copyright (C) 2014-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Volker Krause <volker.krause@kdab.com>
 
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  accordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
@@ -43,6 +48,16 @@ DebuggerInjector::~DebuggerInjector()
 {
 }
 
+void DebuggerInjector::stop()
+{
+    if (m_process) {
+        m_process->terminate();
+        if (!m_process->waitForFinished(1000))
+            m_process->kill(); // kill it softly
+    }
+    emit finished();
+}
+
 QString DebuggerInjector::errorString()
 {
   return mErrorString;
@@ -65,38 +80,60 @@ QProcess::ProcessError DebuggerInjector::processError()
 
 void DebuggerInjector::readyReadStandardOutput()
 {
+  emit stdoutMessage(m_process->readAllStandardOutput());
+}
+
+void DebuggerInjector::processFinished()
+{
+    mExitCode = m_process->exitCode();
+    mExitStatus = m_process->exitStatus();
+    if (!mManualError) {
+      mProcessError = m_process->error();
+      mErrorString = m_process->errorString();
+    }
+    emit attached();
 }
 
 void DebuggerInjector::readyReadStandardError()
 {
   const QString error = m_process->readAllStandardError();
   std::cerr << qPrintable(error) << std::endl;
+  emit stderrMessage(error);
 }
 
-bool DebuggerInjector::startDebugger(const QStringList& args)
+bool DebuggerInjector::startDebugger(const QStringList& args, const QProcessEnvironment &env)
 {
   m_process.reset(new QProcess);
+  if (!env.isEmpty())
+    m_process->setProcessEnvironment(env);
   connect(m_process.data(), SIGNAL(readyReadStandardError()),
           this, SLOT(readyReadStandardError()));
   connect(m_process.data(), SIGNAL(readyReadStandardOutput()),
           this, SLOT(readyReadStandardOutput()));
+  connect(m_process.data(), SIGNAL(started()),
+          this, SIGNAL(started()));
+  connect(m_process.data(), SIGNAL(finished(int)),
+          this, SLOT(processFinished()));
   m_process->setProcessChannelMode(QProcess::SeparateChannels);
   m_process->start(debuggerExecutable(), args);
   bool status = m_process->waitForStarted(-1);
 
-  mExitCode = m_process->exitCode();
-  mExitStatus = m_process->exitStatus();
-  if (!mManualError) {
-    mProcessError = m_process->error();
-    mErrorString = m_process->errorString();
+  if (!status) {
+    mExitCode = m_process->exitCode();
+    mExitStatus = m_process->exitStatus();
+    if (!mManualError) {
+      mProcessError = m_process->error();
+      mErrorString = m_process->errorString();
+    }
+  } else {
+      emit started();
   }
-
   return status;
 }
 
 bool DebuggerInjector::selfTest()
 {
-  if (startDebugger(QStringList() << QLatin1String("--version"))) {
+  if (startDebugger(QStringList() << QStringLiteral("--version"))) {
     return m_process->waitForFinished(-1);
   }
   return false;
@@ -112,42 +149,33 @@ void DebuggerInjector::waitForMain()
 #else
   loadSymbols("Qt5Core");
 #endif
-  // either this
   addMethodBreakpoint("QCoreApplication::exec");
-  // or this for unit tests should hit
-  addMethodBreakpoint("QTest::qExec");
   execCmd("continue");
 }
 
-int DebuggerInjector::injectAndDetach(const QString &probeDll, const QString &probeFunc)
+bool DebuggerInjector::injectAndDetach(const QString &probeDll, const QString &probeFunc)
 {
   Q_ASSERT(m_process);
   loadSymbols("dl");
-  execCmd(QString::fromLatin1("call (void) dlopen(\"%1\", %2)").
+  execCmd(QStringLiteral("call (void) dlopen(\"%1\", %2)").
           arg(probeDll).arg(RTLD_NOW).toUtf8());
   loadSymbols(probeDll.toUtf8());
-  execCmd(QString::fromLatin1("call (void) %1()").arg(probeFunc).toUtf8());
+  execCmd(QStringLiteral("call (void) %1()").arg(probeFunc).toUtf8());
 
   if (qgetenv("GAMMARAY_UNITTEST") != "1") {
     execCmd("detach");
     execCmd("quit");
   } else {
+    // delete all breakpoints before we continue, so we don't hit another one and abort there
+    execCmd("delete");
     execCmd("continue");
     // if we hit a crash or anything, print backtrace and quit
     execCmd("backtrace", false);
     execCmd("quit", false);
+
   }
 
-  m_process->waitForFinished(-1);
-
-  mExitCode = m_process->exitCode();
-  mExitStatus = m_process->exitStatus();
-  if (!mManualError) {
-    mProcessError = m_process->error();
-    mErrorString = m_process->errorString();
-  }
-
-  return mExitCode == EXIT_SUCCESS && mExitStatus == QProcess::NormalExit;
+  return true;
 }
 
 void DebuggerInjector::loadSymbols(const QByteArray& library)
