@@ -4,9 +4,14 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2010-2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Copyright (C) 2010-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Volker Krause <volker.krause@kdab.com>
 
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  accordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
@@ -24,22 +29,26 @@
 #ifndef GAMMARAY_INJECTORFACTORY_H
 #define GAMMARAY_INJECTORFACTORY_H
 
+#include "gammaray_launcher_export.h"
+
 #include "abstractinjector.h"
 
 namespace GammaRay {
 
+class ProbeABI;
+
 namespace InjectorFactory {
 
-   AbstractInjector::Ptr createInjector(const QString &name);
+   GAMMARAY_LAUNCHER_EXPORT AbstractInjector::Ptr createInjector(const QString &name);
 
-   AbstractInjector::Ptr defaultInjectorForLaunch();
+   AbstractInjector::Ptr defaultInjectorForLaunch(const ProbeABI &abi);
 
    AbstractInjector::Ptr defaultInjectorForAttach();
 
    /**
     * Returns the list of available injector types.
     */
-   QStringList availableInjectors();
+   GAMMARAY_LAUNCHER_EXPORT QStringList availableInjectors();
 }
 
 }
