@@ -4,9 +4,14 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2010-2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Copyright (C) 2010-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Volker Krause <volker.krause@kdab.com>
 
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  accordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
@@ -30,18 +35,16 @@ namespace GammaRay {
 
 class StyleInjector : public GammaRay::ProcessInjector
 {
+  Q_OBJECT
   public:
     StyleInjector();
 
-    QString name() const
-    {
-      return QString("style");
-    }
+    QString name() const Q_DECL_OVERRIDE;
+    bool launch(const QStringList &programAndArgs,
+                const QString &probeDll, const QString &probeFunc,
+                const QProcessEnvironment &env) Q_DECL_OVERRIDE;
 
-    virtual bool launch(const QStringList &programAndArgs,
-                        const QString &probeDll, const QString &probeFunc);
-
-    virtual bool selfTest();
+    bool selfTest() Q_DECL_OVERRIDE;
 };
 
 }
