@@ -4,9 +4,14 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2013-2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Copyright (C) 2013-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Volker Krause <volker.krause@kdab.com>
 
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  accordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
@@ -24,6 +29,8 @@
 #ifndef GAMMARAY_LAUNCHERFINDER_H
 #define GAMMARAY_LAUNCHERFINDER_H
 
+#include "gammaray_launcher_export.h"
+
 #include <QString>
 
 namespace GammaRay {
@@ -37,7 +44,7 @@ namespace LauncherFinder
     Client
   };
 
-  QString findLauncher(Type type);
+  GAMMARAY_LAUNCHER_EXPORT QString findLauncher(Type type);
 }
 
 }
