@@ -4,9 +4,14 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2013-2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Copyright (C) 2013-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Volker Krause <volker.krause@kdab.com>
 
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  accordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
@@ -22,6 +27,7 @@
 */
 
 #include "launchoptions.h"
+#include "probeabi.h"
 
 #include <QVariant>
 #include <QProcess>
@@ -30,11 +36,32 @@
 #include <QStandardPaths>
 #endif
 
+namespace GammaRay {
+
+class LaunchOptionsPrivate : public QSharedData
+{
+public:
+    LaunchOptionsPrivate() :
+        pid(-1),
+        uiMode(LaunchOptions::OutOfProcessUi),
+        env(QProcessEnvironment::systemEnvironment())
+    {}
+
+    QStringList launchArguments;
+    QString injectorType;
+    ProbeABI probeABI;
+    int pid;
+    LaunchOptions::UiMode uiMode;
+    QHash<QByteArray, QByteArray> probeSettings;
+    QProcessEnvironment env;
+};
+
+}
+
 using namespace GammaRay;
 
 LaunchOptions::LaunchOptions() :
-  m_pid(-1),
-  m_uiMode(OutOfProcessUi)
+    d(new LaunchOptionsPrivate)
 {
 }
 
@@ -42,9 +69,20 @@ LaunchOptions::~LaunchOptions()
 {
 }
 
+LaunchOptions::LaunchOptions(const LaunchOptions& other) :
+    d(other.d)
+{
+}
+
+LaunchOptions& LaunchOptions::operator=(const LaunchOptions& other)
+{
+    d = other.d;
+    return *this;
+}
+
 bool LaunchOptions::isLaunch() const
 {
-  return !m_launchArguments.isEmpty();
+  return !d->launchArguments.isEmpty();
 }
 
 bool LaunchOptions::isAttach() const
@@ -59,73 +97,106 @@ bool LaunchOptions::isValid() const
 
 QStringList LaunchOptions::launchArguments() const
 {
-  return m_launchArguments;
+  return d->launchArguments;
 }
 
 void LaunchOptions::setLaunchArguments(const QStringList& args)
 {
-  m_launchArguments = args;
-  Q_ASSERT(m_pid <= 0 || m_launchArguments.isEmpty());
+  d->launchArguments = args;
+  Q_ASSERT(d->pid <= 0 || d->launchArguments.isEmpty());
 }
 
 QString LaunchOptions::absoluteExecutablePath() const
 {
-  if (m_launchArguments.isEmpty())
+  if (d->launchArguments.isEmpty())
     return QString();
 
 #if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
-  QString path = m_launchArguments.first();
+  QString path = d->launchArguments.first();
   const QFileInfo fi(path);
   if (fi.isFile() && fi.isExecutable())
     return path;
-  path = QStandardPaths::findExecutable(m_launchArguments.first());
+  path = QStandardPaths::findExecutable(d->launchArguments.first());
   if (!path.isEmpty())
     return path;
 #endif
 
-  return m_launchArguments.first();
+  return d->launchArguments.first();
 }
 
 int LaunchOptions::pid() const
 {
-  return m_pid;
+  return d->pid;
 }
 
 void LaunchOptions::setPid(int pid)
 {
-  m_pid = pid;
-  Q_ASSERT(m_pid <= 0 || m_launchArguments.isEmpty());
+  d->pid = pid;
+  Q_ASSERT(d->pid <= 0 || d->launchArguments.isEmpty());
 }
 
 LaunchOptions::UiMode LaunchOptions::uiMode() const
 {
-  return m_uiMode;
+  return d->uiMode;
 }
 
 void LaunchOptions::setUiMode(LaunchOptions::UiMode mode)
 {
-  m_uiMode = mode;
-  setProbeSetting("InProcessUi", mode == InProcessUi);
+  d->uiMode = mode;
+  setProbeSetting(QStringLiteral("InProcessUi"), mode == InProcessUi);
 }
 
 QString LaunchOptions::injectorType() const
 {
-  return m_injectorType;
+  return d->injectorType;
 }
 
 void LaunchOptions::setInjectorType(const QString& injectorType)
 {
-  m_injectorType = injectorType;
+  d->injectorType = injectorType;
 }
 
 ProbeABI LaunchOptions::probeABI() const
 {
-  return m_probeABI;
+  return d->probeABI;
 }
 
 void LaunchOptions::setProbeABI(const ProbeABI& abi)
 {
-  m_probeABI = abi;
+  d->probeABI = abi;
+}
+
+void LaunchOptions::setRootPath(const QString &path)
+{
+    setProbeSetting(QStringLiteral("RootPath"), path);
+}
+
+QString LaunchOptions::rootPath() const
+{
+    return d->probeSettings.value("RootPath");
+}
+
+void LaunchOptions::setProcessEnvironment(const QProcessEnvironment &env)
+{
+    d->env = env;
+}
+
+QProcessEnvironment LaunchOptions::processEnvironment() const
+{
+    QProcessEnvironment env = d->env;
+    for (auto it = d->probeSettings.constBegin(); it!= d->probeSettings.constEnd(); ++it)
+        env.insert("GAMMARAY_" + it.key(), it.value());
+    return env;
+}
+
+void LaunchOptions::setProbePath(const QString& path)
+{
+  setProbeSetting(QStringLiteral("ProbePath"), path);
+}
+
+QString LaunchOptions::probePath() const
+{
+    return d->probeSettings.value("ProbePath");
 }
 
 void LaunchOptions::setProbeSetting(const QString& key, const QVariant& value)
@@ -139,18 +210,21 @@ void LaunchOptions::setProbeSetting(const QString& key, const QVariant& value)
       v = value.toBool() ? "true" : "false";
       break;
     case QVariant::Int:
+    case QVariant::UInt:
+    case QVariant::LongLong:
+    case QVariant::ULongLong:
       v = QByteArray::number(value.toInt());
       break;
     default:
       qFatal("unsupported probe settings type");
   }
 
-  m_probeSettings.insert(key.toUtf8(), v);
+  d->probeSettings.insert(key.toUtf8(), v);
 }
 
 QHash< QByteArray, QByteArray > LaunchOptions::probeSettings() const
 {
-  return m_probeSettings;
+  return d->probeSettings;
 }
 
 bool LaunchOptions::execute(const QString& launcherPath) const
@@ -161,30 +235,30 @@ bool LaunchOptions::execute(const QString& launcherPath) const
   QStringList args;
   switch (uiMode()) {
     case InProcessUi:
-      args.push_back("--inprocess");
+      args.push_back(QStringLiteral("--inprocess"));
       break;
     case OutOfProcessUi:
-      args.push_back("--no-inprocess");
+      args.push_back(QStringLiteral("--no-inprocess"));
       break;
     case NoUi:
-      args.push_back("--inject-only");
+      args.push_back(QStringLiteral("--inject-only"));
       break;
   }
 
-  if (m_probeABI.isValid()) {
-    args.push_back("--probe");
-    args.push_back(m_probeABI.id());
+  if (d->probeABI.isValid()) {
+    args.push_back(QStringLiteral("--probe"));
+    args.push_back(d->probeABI.id());
   }
 
-  if (m_probeSettings.contains("TCPServer")) {
-    args.push_back("--listen");
-    args.push_back(m_probeSettings.value("TCPServer"));
+  if (d->probeSettings.contains("ServerAddress")) {
+    args.push_back(QStringLiteral("--listen"));
+    args.push_back(d->probeSettings.value("ServerAddress"));
   }
-  if (m_probeSettings.value("RemoteAccessEnabled") == "false")
-    args.push_back("--no-listen");
+  if (d->probeSettings.value("RemoteAccessEnabled") == "false")
+    args.push_back(QStringLiteral("--no-listen"));
 
   if (isAttach()) {
-    args.push_back("--pid");
+    args.push_back(QStringLiteral("--pid"));
     args.push_back(QString::number(pid()));
   } else {
     args += launchArguments();
