@@ -4,9 +4,14 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2010-2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Copyright (C) 2010-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Volker Krause <volker.krause@kdab.com>
 
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  accordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
@@ -21,17 +26,18 @@
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include "config-gammaray.h"
+#include <config-gammaray.h>
 #include "config-gammaray-version.h"
 #include "injector/injectorfactory.h"
+#include "clientlauncher.h"
 #include "launchoptions.h"
 #include "launcherfinder.h"
 #include "launcher.h"
 #include "probefinder.h"
 
 #include <common/paths.h>
-#include <common/probeabi.h>
-#include <common/probeabidetector.h>
+#include <launcher/probeabi.h>
+#include <launcher/probeabidetector.h>
 
 #ifdef HAVE_QT_WIDGETS
 #include <QApplication>
@@ -41,9 +47,44 @@
 
 #include <QDebug>
 #include <QDir>
+#include <QUrl>
 #include <QStringList>
 #include <QVariant>
 
+#include <csignal>
+
+namespace {
+void shutdownGracefully(int sig)
+{
+    static volatile std::sig_atomic_t handlingSignal = 0;
+
+    if ( !handlingSignal ) {
+        handlingSignal = 1;
+        qDebug() << "Signal" << sig << "received, shutting down gracefully.";
+        QCoreApplication* app = QCoreApplication::instance();
+        app->quit();
+        return;
+    }
+
+    // re-raise signal with default handler and trigger program termination
+    std::signal(sig, SIG_DFL);
+    std::raise(sig);
+}
+
+void installSignalHandler()
+{
+#ifdef SIGHUP
+    std::signal(SIGHUP, shutdownGracefully);
+#endif
+#ifdef SIGINT
+    std::signal(SIGINT, shutdownGracefully);
+#endif
+#ifdef SIGTERM
+    std::signal(SIGTERM, shutdownGracefully);
+#endif
+}
+}
+
 using namespace GammaRay;
 
 QTextStream out(stdout);
@@ -61,7 +102,7 @@ static void usage(const char *argv0)
   out << "" << endl;
   out << "Options:" << endl;
   out << " -i, --injector <injector>  \tset injection type, possible values:" << endl;
-  out << "                            \t" << InjectorFactory::availableInjectors().join(", ")
+  out << "                            \t" << InjectorFactory::availableInjectors().join(QStringLiteral(", "))
       << endl;
   out << " -p, --pid <pid>            \tattach to running Qt application" << endl;
   out << "     --inprocess            \tuse in-process UI" << endl;
@@ -94,13 +135,36 @@ static bool startLauncher()
   return proc.exitCode() == 0;
 }
 
+static QUrl urlFromUserInput(const QString &s)
+{
+  QUrl url(s);
+  if (url.scheme().isEmpty()) { // backward compat: map input without a scheme to tcp + hostname
+    url.setScheme(QStringLiteral("tcp"));
+    QString host = url.path();
+    int port = -1;
+    const int pos = host.lastIndexOf(':');
+    if (pos > 0) {
+      port = host.mid(pos + 1).toUShort();
+      host = host.left(pos);
+    }
+    url.setHost(host);
+    url.setPort(port);
+    url.setPath(QString());
+  }
+
+  return url;
+}
+
 int main(int argc, char **argv)
 {
-  QCoreApplication::setOrganizationName("KDAB");
-  QCoreApplication::setOrganizationDomain("kdab.com");
-  QCoreApplication::setApplicationName("GammaRay");
+  QCoreApplication::setOrganizationName(QStringLiteral("KDAB"));
+  QCoreApplication::setOrganizationDomain(QStringLiteral("kdab.com"));
+  QCoreApplication::setApplicationName(QStringLiteral("GammaRay"));
+
+  installSignalHandler();
 
   QStringList args;
+  args.reserve(argc);
   for (int i = 1; i < argc; ++i) {
     args.push_back(QString::fromLocal8Bit(argv[i]));
   }
@@ -111,9 +175,9 @@ int main(int argc, char **argv)
 #endif
   Paths::setRelativeRootPath(GAMMARAY_INVERSE_BIN_DIR);
 
-  QStringList builtInArgs = QStringList() << QLatin1String("-style")
-                                          << QLatin1String("-stylesheet")
-                                          << QLatin1String("-graphicssystem");
+  QStringList builtInArgs = QStringList() << QStringLiteral("-style")
+                                          << QStringLiteral("-stylesheet")
+                                          << QStringLiteral("-graphicssystem");
 
   LaunchOptions options;
   while (!args.isEmpty() && args.first().startsWith('-')) {
@@ -132,7 +196,7 @@ int main(int argc, char **argv)
     }
     if (arg == QLatin1String("-v") || arg == QLatin1String("--version")) {
       out << "GammaRay version " << GAMMARAY_VERSION_STRING << endl;
-      out << "Copyright (C) 2010-2014 Klaralvdalens Datakonsult AB, "
+      out << "Copyright (C) 2010-2015 Klaralvdalens Datakonsult AB, "
           << "a KDAB Group company, info@kdab.com" << endl;
       return 0;
     }
@@ -143,10 +207,10 @@ int main(int argc, char **argv)
       options.setUiMode(LaunchOptions::NoUi);
     }
     if (arg == QLatin1String("--listen") && !args.isEmpty()) {
-      options.setProbeSetting("TCPServer", args.takeFirst());
+      options.setProbeSetting(QStringLiteral("ServerAddress"), urlFromUserInput(args.takeFirst()).toString());
     }
     if ( arg == QLatin1String("--no-listen")) {
-      options.setProbeSetting("RemoteAccessEnabled", false);
+      options.setProbeSetting(QStringLiteral("RemoteAccessEnabled"), false);
       options.setUiMode(LaunchOptions::InProcessUi);
     }
     if ( arg == QLatin1String("--list-probes")) {
@@ -160,22 +224,16 @@ int main(int argc, char **argv)
         out << "Invalid probe ABI specified, see --list-probes for valid ones." << endl;
         return 1;
       }
-      if (ProbeFinder::findProbe("gammaray_probe", abi).isEmpty()) {
+      if (ProbeFinder::findProbe(QStringLiteral(GAMMARAY_PROBE_NAME), abi).isEmpty()) {
         out << abi.id() << "is not a known probe, see --list-probes." << endl;
         return 1;
       }
       options.setProbeABI(abi);
     }
     if ( arg == QLatin1String("--connect") && !args.isEmpty()) {
-      QString host = args.takeFirst();
-      quint16 port = 0;
-      const int pos = host.lastIndexOf(":");
-      if (pos > 0) {
-        port = host.mid(pos + 1).toUShort();
-        host = host.left(pos);
-      }
+      const QUrl url = urlFromUserInput(args.takeFirst());
       ClientLauncher client;
-      client.launch(host, port);
+      client.launch(url);
       client.waitForFinished();
       return 0;
     }
@@ -243,5 +301,10 @@ int main(int argc, char **argv)
   }
 
   Launcher launcher(options);
-  return app.exec();
+  QObject::connect(&launcher, SIGNAL(finished()), &app, SLOT(quit()));
+  QObject::connect(&launcher, SIGNAL(attached()), &app, SLOT(quit()));
+  if (!launcher.start())
+    return launcher.exitCode();
+  auto result = app.exec();
+  return result == 0 ? launcher.exitCode() : result;
 }
