@@ -4,9 +4,14 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2010-2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Copyright (C) 2010-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Milian Wolff <milian.wolff@kdab.com>
 
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  accordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
@@ -28,6 +33,8 @@
 #include "processmodel.h"
 #include "probeabimodel.h"
 
+#include <ui/searchlinecontroller.h>
+
 #include <QPushButton>
 #include <QStandardItemModel>
 #include <QListView>
@@ -45,6 +52,12 @@ AttachDialog::AttachDialog(QWidget *parent, Qt::WindowFlags f)
   m_abiModel(new ProbeABIModel(this))
 {
   ui.setupUi(this);
+#if defined(Q_OS_MAC)
+  QMargins margins = ui.formLayout->contentsMargins();
+  margins.setRight(margins.right() +2);
+  margins.setBottom(margins.bottom() +2);
+  ui.formLayout->setContentsMargins(margins);
+#endif
 
   m_model = new ProcessModel(this);
 
@@ -69,19 +82,15 @@ AttachDialog::AttachDialog(QWidget *parent, Qt::WindowFlags f)
 
   connect(ui.view, SIGNAL(activated(QModelIndex)), SIGNAL(activate()));
 
-  ui.filter->setProxy(m_proxyModel);
+  new SearchLineController(ui.filter, m_proxyModel);
 
   ui.probeBox->setModel(m_abiModel);
 
   QSettings settings;
-  ui.accessMode->setCurrentIndex(settings.value(QLatin1String("Launcher/AttachAccessMode")).toInt());
+  ui.accessMode->setCurrentIndex(settings.value(QStringLiteral("Launcher/AttachAccessMode")).toInt());
 
   setWindowTitle(tr("GammaRay - Attach to Process"));
-  setWindowIcon(QIcon(":gammaray/GammaRay-128x128.png"));
-
-  m_timer = new QTimer(this);
-  connect(m_timer, SIGNAL(timeout()), this, SLOT(updateProcesses()));
-  m_timer->start(1000);
+  setWindowIcon(QIcon(QStringLiteral(":gammaray/GammaRay-128x128.png")));
 
   ui.stackedWidget->setCurrentWidget(ui.loadingLabel);
   emit updateButtonState();
@@ -96,7 +105,7 @@ bool AttachDialog::isValid() const
 void AttachDialog::writeSettings()
 {
   QSettings settings;
-  settings.setValue(QLatin1String("Launcher/AttachAccessMode"), ui.accessMode->currentIndex());
+  settings.setValue(QStringLiteral("Launcher/AttachAccessMode"), ui.accessMode->currentIndex());
 }
 
 LaunchOptions AttachDialog::launchOptions() const
@@ -107,15 +116,15 @@ LaunchOptions AttachDialog::launchOptions() const
 
   switch (ui.accessMode->currentIndex()) {
     case 0: // local, out-of-process
-      opt.setProbeSetting("TCPServer", "127.0.0.1");
+      opt.setProbeSetting(QStringLiteral("ServerAddress"), "tcp://127.0.0.1/");
       opt.setUiMode(LaunchOptions::OutOfProcessUi);
       break;
     case 1: // remote, out-of-process
-      opt.setProbeSetting("TCPServer", "0.0.0.0");
+      opt.setProbeSetting(QStringLiteral("ServerAddress"), "tcp://0.0.0.0/");
       opt.setUiMode(LaunchOptions::OutOfProcessUi);
       break;
     case 2: // in-process
-      opt.setProbeSetting("RemoteAccessEnabled", false);
+      opt.setProbeSetting(QStringLiteral("RemoteAccessEnabled"), false);
       opt.setUiMode(LaunchOptions::InProcessUi);
       break;
   }
@@ -147,6 +156,8 @@ void AttachDialog::updateProcessesFinished()
     ui.view->setCurrentIndex(QModelIndex());
   }
   watcher->deleteLater();
+
+  QTimer::singleShot(1000, this, SLOT(updateProcesses()));
 }
 
 void AttachDialog::selectABI(const QModelIndex& processIndex)
