@@ -4,9 +4,14 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2011-2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Copyright (C) 2011-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Volker Krause <volker.krause@kdab.com>
 
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  accordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
@@ -43,6 +48,12 @@ LaunchPage::LaunchPage(QWidget *parent)
     m_abiIsValid(true)
 {
   ui->setupUi(this);
+#if defined(Q_OS_MAC)
+  QMargins margins = ui->formLayout->contentsMargins();
+  margins.setRight(margins.right() +2);
+  margins.setBottom(margins.bottom() +2);
+  ui->formLayout->setContentsMargins(margins);
+#endif
   connect(ui->progSelectButton, SIGNAL(clicked()), SLOT(showFileDialog()));
   connect(ui->addArgButton, SIGNAL(clicked()), SLOT(addArgument()));
   connect(ui->removeArgButton, SIGNAL(clicked()), SLOT(removeArgument()));
@@ -60,9 +71,9 @@ LaunchPage::LaunchPage(QWidget *parent)
   ui->probeBox->setModel(m_abiModel);
 
   QSettings settings;
-  ui->progEdit->setText(settings.value(QLatin1String("Launcher/Program")).toString());
-  m_argsModel->setStringList(settings.value(QLatin1String("Launcher/Arguments")).toStringList());
-  ui->accessMode->setCurrentIndex(settings.value(QLatin1String("Launcher/AccessMode")).toInt());
+  ui->progEdit->setText(settings.value(QStringLiteral("Launcher/Program")).toString());
+  m_argsModel->setStringList(settings.value(QStringLiteral("Launcher/Arguments")).toStringList());
+  ui->accessMode->setCurrentIndex(settings.value(QStringLiteral("Launcher/AccessMode")).toInt());
   updateArgumentButtons();
 }
 
@@ -74,9 +85,9 @@ LaunchPage::~LaunchPage()
 void LaunchPage::writeSettings()
 {
   QSettings settings;
-  settings.setValue(QLatin1String("Launcher/Program"), ui->progEdit->text());
-  settings.setValue(QLatin1String("Launcher/Arguments"), notEmptyString(m_argsModel->stringList()));
-  settings.setValue(QLatin1String("Launcher/AccessMode"), ui->accessMode->currentIndex());
+  settings.setValue(QStringLiteral("Launcher/Program"), ui->progEdit->text());
+  settings.setValue(QStringLiteral("Launcher/Arguments"), notEmptyString(m_argsModel->stringList()));
+  settings.setValue(QStringLiteral("Launcher/AccessMode"), ui->accessMode->currentIndex());
 }
 
 QStringList LaunchPage::notEmptyString(const QStringList &list) const
@@ -103,17 +114,17 @@ LaunchOptions LaunchPage::launchOptions() const
 
   switch (ui->accessMode->currentIndex()) {
     case 0: // local, out-of-process
-      opt.setProbeSetting("RemoteAccessEnabled", true);
-      opt.setProbeSetting("TCPServer", "127.0.0.1");
+      opt.setProbeSetting(QStringLiteral("RemoteAccessEnabled"), true);
+      opt.setProbeSetting(QStringLiteral("ServerAddress"), "tcp://127.0.0.1/");
       opt.setUiMode(LaunchOptions::OutOfProcessUi);
       break;
     case 1: // remote, out-of-process
-      opt.setProbeSetting("RemoteAccessEnabled", true);
-      opt.setProbeSetting("TCPServer", "0.0.0.0");
+      opt.setProbeSetting(QStringLiteral("RemoteAccessEnabled"), true);
+      opt.setProbeSetting(QStringLiteral("ServerAddress"), "tcp://0.0.0.0/");
       opt.setUiMode(LaunchOptions::OutOfProcessUi);
       break;
     case 2: // in-process
-      opt.setProbeSetting("RemoteAccessEnabled", false);
+      opt.setProbeSetting(QStringLiteral("RemoteAccessEnabled"), false);
       opt.setUiMode(LaunchOptions::InProcessUi);
       break;
   }
