@@ -4,9 +4,14 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2010-2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Copyright (C) 2010-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Milian Wolff <milian.wolff@kdab.com>
 
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  accordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
@@ -173,12 +178,13 @@ QVariant ProcessModel::data(const QModelIndex &index, int role) const
     }
   } else if (role == Qt::ToolTipRole) {
     const ProbeABI bestABI = ProbeFinder::findBestMatchingABI(data.abi, m_availableABIs);
-    return tr("Name: %1\nPID: %2\nOwner: %3\nQt ABI: %4\nProbe available: %5")
-      .arg(data.image.isEmpty() ? data.name : data.image)
-      .arg(data.ppid)
-      .arg(data.user)
-      .arg(data.abi.displayString())
-      .arg(bestABI.isValid() ? tr("yes") : tr("no"));
+    return tr("Name: %1\nPID: %2\nOwner: %3\nQt ABI: %4\nProbe available: %5").arg(
+      data.image.isEmpty() ? data.name : data.image,
+      data.ppid,
+      data.user,
+      data.abi.displayString(),
+      bestABI.isValid() ? tr("yes") : tr("no")
+    );
   } else if (role == PIDRole) {
     return data.ppid.toInt(); // why is this a QString in the first place!?
   } else if (role == NameRole) {
