@@ -4,9 +4,14 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2011-2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Copyright (C) 2011-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Volker Krause <volker.krause@kdab.com>
 
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  accordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
@@ -21,13 +26,15 @@
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include <config-gammaray.h>
+
 #include "selftestpage.h"
 #include "probefinder.h"
 #include "ui_selftestpage.h"
 
 #include "injector/injectorfactory.h"
 
-#include <common/probeabi.h>
+#include <launcher/probeabi.h>
 
 #include <QFileInfo>
 #include <QStandardItemModel>
@@ -60,7 +67,7 @@ void SelfTestPage::testProbe()
   int validProbeCount = 0;
   const QVector<ProbeABI> probeABIs = ProbeFinder::listProbeABIs();
   foreach (const ProbeABI &abi, probeABIs) {
-    const QString probePath = ProbeFinder::findProbe(QLatin1String("gammaray_probe"), abi);
+    const QString probePath = ProbeFinder::findProbe(QStringLiteral(GAMMARAY_PROBE_NAME), abi);
     if (probePath.isEmpty()) {
       error(tr("No probe found for ABI %1.").arg(abi.id()));
       continue;
@@ -72,12 +79,12 @@ void SelfTestPage::testProbe()
       continue;
     }
 
-    information(tr("Found valid probe for ABI %1 at %2.").arg(abi.id()).arg(probePath));
+    information(tr("Found valid probe for ABI %1 at %2.").arg(abi.id(), probePath));
     ++validProbeCount;
   }
 
   if (validProbeCount == 0) {
-    error(tr("No probes found - GammaRay no functional."));
+    error(tr("No probes found - GammaRay not functional."));
   }
 }
 
@@ -90,7 +97,7 @@ void SelfTestPage::testAvailableInjectors()
   }
 
   information(tr("The following injectors are available: %1").
-              arg(injectors.join(QLatin1String(", "))));
+              arg(injectors.join(QStringLiteral(", "))));
 }
 
 void SelfTestPage::testInjectors()
