@@ -2,9 +2,14 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2010-2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Copyright (C) 2010-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Kevin Funk <kevin.funk@kdab.com>
 
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  accordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
@@ -25,8 +30,8 @@
 #include <QAction>
 #include <QWidget>
 
-class QAbstractProxyModel;
-class QTreeView;
+#include <ui/tooluifactory.h>
+
 class QModelIndex;
 
 namespace GammaRay {
@@ -41,10 +46,13 @@ class ActionInspectorWidget : public QWidget
 
   private Q_SLOTS:
     void triggerAction(const QModelIndex &index);
+};
 
-  private:
-    QTreeView *mObjectTreeView;
-    QAbstractProxyModel *m_proxy;
+class ActionInspectorUiFactory : public QObject, public StandardToolUiFactory<ActionInspectorWidget>
+{
+  Q_OBJECT
+  Q_INTERFACES(GammaRay::ToolUiFactory)
+  Q_PLUGIN_METADATA(IID "com.kdab.GammaRay.ToolUiFactory" FILE "gammaray_actioninspector.json")
 };
 
 }
