@@ -4,9 +4,14 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2010-2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Copyright (C) 2010-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Stephen Kelly <stephen.kelly@kdab.com>
 
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  accordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
@@ -30,7 +35,7 @@
 using namespace GammaRay;
 
 FontModel::FontModel(QObject *parent)
-  : QAbstractItemModel(parent)
+  : QAbstractTableModel(parent)
   , m_size(12)
   , m_bold(false)
   , m_italic(false)
@@ -38,13 +43,25 @@ FontModel::FontModel(QObject *parent)
 {
 }
 
-QList<QFont> FontModel::currentFonts() const
+QVector<QFont> FontModel::currentFonts() const
 {
   return m_fonts;
 }
 
-void FontModel::updateFonts(const QList<QFont> &fonts)
+void FontModel::updateFonts(const QVector<QFont> &fonts)
 {
+  if (!m_fonts.isEmpty()) {
+    beginRemoveRows(QModelIndex(), 0, m_fonts.size() - 1);
+    m_fonts.clear();
+    endRemoveRows();
+  }
+
+  if (fonts.isEmpty())
+    return;
+
+  beginInsertRows(QModelIndex(), 0, fonts.size() - 1);
+  m_fonts = fonts;
+
   for (int i = 0; i < m_fonts.size(); ++i) {
     QFont &font = m_fonts[i];
     font.setPointSize(m_size);
@@ -53,9 +70,7 @@ void FontModel::updateFonts(const QList<QFont> &fonts)
     font.setUnderline(m_underline);
   }
 
-  beginResetModel();
-  m_fonts = fonts;
-  endResetModel();
+  endInsertRows();
 }
 
 void FontModel::updateText(const QString &text)
@@ -63,9 +78,8 @@ void FontModel::updateText(const QString &text)
   if (text == m_text) {
     return;
   }
-  beginResetModel();
   m_text = text;
-  endResetModel();
+  fontDataChanged();
 }
 
 QVariant FontModel::headerData(int section, Qt::Orientation orientation, int role) const
@@ -79,7 +93,7 @@ QVariant FontModel::headerData(int section, Qt::Orientation orientation, int rol
       return "Text Preview";
     }
   }
-  return QAbstractItemModel::headerData(section, orientation, role);
+  return QAbstractTableModel::headerData(section, orientation, role);
 }
 
 int FontModel::rowCount(const QModelIndex &parent) const
@@ -90,23 +104,6 @@ int FontModel::rowCount(const QModelIndex &parent) const
   return m_fonts.size();
 }
 
-QModelIndex FontModel::index(int row, int column, const QModelIndex &parent) const
-{
-  if (parent.isValid()) {
-    return QModelIndex();
-  }
-  if (!hasIndex(row, column, parent)) {
-    return QModelIndex();
-  }
-  return createIndex(row, column);
-}
-
-QModelIndex FontModel::parent(const QModelIndex &child) const
-{
-  Q_UNUSED(child);
-  return QModelIndex();
-}
-
 int FontModel::columnCount(const QModelIndex &parent) const
 {
   Q_UNUSED(parent);
@@ -121,11 +118,7 @@ QVariant FontModel::data(const QModelIndex &index, int role) const
     }
   } else if (index.column() == 1) {
     if (role == Qt::DisplayRole) {
-#if QT_VERSION >= 0x040800
       return m_fonts.at(index.row()).styleName();
-#else
-      return tr("N/A");
-#endif
     }
   } else if (index.column() == 2) {
     if (role == Qt::DecorationRole || role == Qt::SizeHintRole) {
