@@ -4,9 +4,14 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2012-2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Copyright (C) 2012-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Volker Krause <volker.krause@kdab.com>
 
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  accordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
@@ -143,7 +148,7 @@ void KJobModel::objectRemoved(QObject *obj)
   // We however seem to get here for very short-lived jobs that emit before objectAdded()
   // is called (while we wait for the vtable to be complete), so we only see the result
   // of their deleteLater().
-  if (m_data[pos].state == KJobInfo::Running) {
+  if (m_data.at(pos).state == KJobInfo::Running) {
     m_data[pos].state = KJobInfo::Deleted;
     m_data[pos].statusText = tr("Deleted");
     emit dataChanged(index(pos, 0), index(pos, columnCount() - 1));
