@@ -4,9 +4,14 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2012-2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Copyright (C) 2012-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Volker Krause <volker.krause@kdab.com>
 
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  accordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
@@ -27,6 +32,7 @@
 #include <core/probeinterface.h>
 
 #include <QDebug>
+#include <QSortFilterProxyModel>
 #include <QtPlugin>
 
 using namespace GammaRay;
@@ -39,13 +45,21 @@ KJobTracker::KJobTracker(ProbeInterface *probe, QObject *parent)
   connect(probe->probe(), SIGNAL(objectDestroyed(QObject*)),
           m_jobModel, SLOT(objectRemoved(QObject*)));
 
-  probe->registerModel("com.kdab.GammaRay.KJobModel", m_jobModel);
+  auto proxy = new QSortFilterProxyModel(this);
+  proxy->setSourceModel(m_jobModel);
+
+  probe->registerModel(QStringLiteral("com.kdab.GammaRay.KJobModel"), proxy);
 }
 
 KJobTracker::~KJobTracker()
 {
 }
 
+QString KJobTrackerFactory::name() const
+{
+  return tr("KJobs");
+}
+
 #if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
 Q_EXPORT_PLUGIN(KJobTrackerFactory)
 #endif
