@@ -2,9 +2,14 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2010-2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Copyright (C) 2010-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Kevin Funk <kevin.funk@kdab.com>
 
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  accordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
@@ -22,6 +27,8 @@
 #include "objectvisualizer.h"
 #include "objectvisualizermodel.h"
 
+#include <core/remote/serverproxymodel.h>
+
 #include <QtPlugin>
 
 using namespace GammaRay;
@@ -29,8 +36,11 @@ using namespace GammaRay;
 GraphViewer::GraphViewer(ProbeInterface *probe, QObject *parent)
   : QObject(parent)
 {
-  ObjectVisualizerModel *model = new ObjectVisualizerModel(this);
+  auto model = new ServerProxyModel<ObjectVisualizerModel>(this);
   model->setSourceModel(probe->objectTreeModel());
+  model->addProxyRole(ObjectVisualizerModel::ObjectId);
+  model->addProxyRole(ObjectVisualizerModel::ObjectDisplayName);
+  model->addProxyRole(ObjectVisualizerModel::ClassName);
   probe->registerModel("com.kdab.GammaRay.ObjectVisualizerModel", model);
 }
 
@@ -38,6 +48,11 @@ GraphViewer::~GraphViewer()
 {
 }
 
+QString GraphViewerFactory::name() const
+{
+  return tr("Object Visualizer");
+}
+
 #if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
 Q_EXPORT_PLUGIN(GraphViewerFactory)
 #endif
