@@ -4,9 +4,14 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2013-2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Copyright (C) 2013-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Volker Krause <volker.krause@kdab.com>
 
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  accordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
@@ -24,23 +29,15 @@
 #ifndef GAMMARAY_OBJECTVISUALIZER_OBJECTVISUALIZERMODEL_H
 #define GAMMARAY_OBJECTVISUALIZER_OBJECTVISUALIZERMODEL_H
 
-#include <QtGlobal>
-
-#if QT_VERSION < QT_VERSION_CHECK(4, 8, 0)
-#include <QSortFilterProxyModel>
-typedef QSortFilterProxyModel QIdentityProxyModel;
-#else
-#include <QIdentityProxyModel>
-#endif
-
 #include <common/objectmodel.h>
+#include <3rdparty/kde/krecursivefilterproxymodel.h>
 
 namespace GammaRay {
 
 /** Augment the regular object tree by some information needed for the visualization
  * on the client side.
  */
-class ObjectVisualizerModel : public QIdentityProxyModel
+class ObjectVisualizerModel : public KRecursiveFilterProxyModel
 {
     Q_OBJECT
 public:
@@ -53,9 +50,9 @@ public:
     explicit ObjectVisualizerModel(QObject *parent);
     ~ObjectVisualizerModel();
 
-    QVariant data(const QModelIndex& proxyIndex, int role = Qt::DisplayRole) const;
-    QMap<int, QVariant> itemData(const QModelIndex& index) const;
+    QVariant data(const QModelIndex& proxyIndex, int role = Qt::DisplayRole) const Q_DECL_OVERRIDE;
 };
+
 }
 
 #endif // GAMMARAY_OBJECTVISUALIZERMODEL_H
