@@ -4,9 +4,14 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Copyright (C) 2014-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Volker Krause <volker.krause@kdab.com>
 
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  accordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
@@ -31,7 +36,11 @@ namespace GammaRay {
 /** Model roles shared between client and server. */
 namespace QuickItemModelRole {
   enum Roles {
-    ItemFlags = ObjectModel::UserRole
+    ItemFlags = ObjectModel::UserRole,
+    SourceFileRole,
+    SourceLineRole,
+    SourceColumnRole,
+    ItemEvent
   };
 
   enum ItemFlag {
