@@ -2,9 +2,14 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2013-2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Copyright (C) 2013-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Milian Wolff <milian.wolff@kdab.com>
 
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  accordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
@@ -25,6 +30,7 @@
 #include <QObject>
 #include <QMetaType>
 #include <QDataStream>
+#include <QVector>
 
 class QAbstractState;
 class QAbstractTransition;
@@ -36,9 +42,6 @@ namespace GammaRay {
 // to the meta type system)...
 struct TransitionId
 {
-  TransitionId(const TransitionId &id)
-  : id(id.id)
-  {}
   explicit TransitionId(QAbstractTransition *transition = 0)
   : id(reinterpret_cast<quint64>(transition))
   {}
@@ -63,9 +66,6 @@ inline QDataStream &operator>>(QDataStream &in, TransitionId &value)
 
 struct StateId
 {
-  StateId(const StateId &id)
-  : id(id.id)
-  {}
   explicit StateId(QAbstractState *state= 0)
   : id(reinterpret_cast<quint64>(state))
   {}
@@ -91,7 +91,8 @@ inline QDataStream &operator>>(QDataStream &in, StateId &value)
 enum StateType {
   OtherState,
   FinalState,
-  HistoryState,
+  ShallowHistoryState,
+  DeepHistoryState,
   StateMachineState
 };
 
@@ -109,7 +110,7 @@ inline QDataStream &operator>>(QDataStream &in, StateType &value)
   return in;
 }
 
-typedef QList<StateId> StateMachineConfiguration;
+typedef QVector<StateId> StateMachineConfiguration;
 
 class StateMachineViewerInterface : public QObject
 {
@@ -119,6 +120,7 @@ class StateMachineViewerInterface : public QObject
     virtual ~StateMachineViewerInterface();
 
   public slots:
+    virtual void selectStateMachine(int index) = 0;
     virtual void toggleRunning() = 0;
     virtual void setMaximumDepth(int depth) = 0;
 
@@ -143,6 +145,7 @@ class StateMachineViewerInterface : public QObject
 }
 
 Q_DECLARE_METATYPE(GammaRay::StateId)
+Q_DECLARE_TYPEINFO(GammaRay::StateId, Q_PRIMITIVE_TYPE);
 Q_DECLARE_METATYPE(GammaRay::TransitionId)
 Q_DECLARE_METATYPE(GammaRay::StateMachineConfiguration)
 Q_DECLARE_METATYPE(GammaRay::StateType)
