@@ -2,10 +2,15 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2010-2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Copyright (C) 2010-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Kevin Funk <kevin.funk@kdab.com>
   Author: Milian Wolff <milian.wolff@kdab.com>
 
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  accordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
@@ -58,9 +63,11 @@ class StateMachineViewerWidget : public QWidget
     void clearView();
     void repopulateView();
     void clearGraph();
+    //cppcheck-suppress passedByValue
     void stateAdded(const GammaRay::StateId state, const GammaRay::StateId parent, const bool hasChildren,
                     const QString &label, const GammaRay::StateType type, const bool connectToInitial);
     void stateConfigurationChanged(const GammaRay::StateMachineConfiguration &config);
+    //cppcheck-suppress passedByValue
     void transitionAdded(const GammaRay::TransitionId transition, const GammaRay::StateId source, const GammaRay::StateId target,
                          const QString &label);
     void transitionTriggered(GammaRay::TransitionId transition, const QString &label);
@@ -100,7 +107,7 @@ class StateMachineViewerUiFactory : public QObject, public StandardToolUiFactory
 {
   Q_OBJECT
   Q_INTERFACES(GammaRay::ToolUiFactory)
-  Q_PLUGIN_METADATA(IID "com.kdab.gammaray.StateMachineViewerUi")
+  Q_PLUGIN_METADATA(IID "com.kdab.GammaRay.ToolUiFactory" FILE "gammaray_statemachineviewer.json")
 };
 
 }
