@@ -4,7 +4,12 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2010-2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  accordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
+  Copyright (C) 2010-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Stephen Kelly <stephen.kelly@kdab.com>
 
   This program is free software; you can redistribute it and/or modify
@@ -49,6 +54,15 @@ class StateModelPrivate
                          qq, SLOT(stateConfigurationChanged())));
   }
 
+  void emitDataChangedForState(QAbstractState *state)
+  {
+    const auto left = indexForState(state);
+    const auto right = left.sibling(left.row(), q_ptr->columnCount() - 1);
+    if (!left.isValid() || !right.isValid())
+      return;
+    emit q_ptr->dataChanged(left, right);
+  }
+
   Q_DECLARE_PUBLIC(StateModel)
   StateModel * const q_ptr;
   StateMachineWatcher * const m_stateMachineWatcher;
@@ -117,22 +131,14 @@ QModelIndex StateModelPrivate::indexForState(QAbstractState *state) const
 
 void StateModelPrivate::stateConfigurationChanged()
 {
-  Q_Q(StateModel);
-
   QSet<QAbstractState *> newConfig = m_stateMachine->configuration();
   // states which became active
   foreach (QAbstractState *state, (newConfig - m_lastConfiguration)) {
-    const QModelIndex source = indexForState(state);
-    if (source.isValid()) {
-      q->dataChanged(source, source);
-    }
+    emitDataChangedForState(state);
   }
   // states which became inactive
   foreach (QAbstractState *state, (m_lastConfiguration - newConfig)) {
-    const QModelIndex source = indexForState(state);
-    if (source.isValid()) {
-      q->dataChanged(source, source);
-    }
+    emitDataChangedForState(state);
   }
   m_lastConfiguration = newConfig;
 }
@@ -206,12 +212,13 @@ QVariant StateModel::data(const QModelIndex &index, int role) const
       Q_ASSERT(l.contains(state));
       QStringList nums;
       QList<QAbstractTransition*> trs = state->transitions();
+      nums.reserve(trs.size());
       foreach (QAbstractTransition *t, trs) {
         QAbstractState *child = t->targetState();
         Q_ASSERT(l.contains(child));
         nums << QString::number(l.indexOf(child) - l.indexOf(state));
       }
-      return nums.join(",");
+      return nums.join(QStringLiteral(","));
     }
   }
   if (role == IsInitialStateRole) {
@@ -289,4 +296,15 @@ QModelIndex StateModel::parent(const QModelIndex &index) const
   return createIndex(row, 0, grandParent);
 }
 
+QVariant StateModel::headerData(int section, Qt::Orientation orientation, int role) const
+{
+  if (role == Qt::DisplayRole && orientation == Qt::Horizontal) {
+    switch (section) {
+      case 0: return tr("State");
+      case 1: return tr("Type");
+    }
+  }
+  return QAbstractItemModel::headerData(section, orientation, role);
+}
+
 #include "moc_statemodel.cpp"
