@@ -4,9 +4,14 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2010-2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Copyright (C) 2010-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Thomas McGuire <thomas.mcguire@kdab.com>
 
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  accordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
@@ -31,13 +36,17 @@ using namespace GammaRay;
 static const int maxTimeoutEvents = 1000;
 static const int maxTimeSpan = 10000;
 
-TimerInfo::TimerInfo(QTimer *timer)
-  : m_type(QTimerType),
+TimerInfo::TimerInfo(QObject* timer) :
+    m_type(QQmlTimerType),
     m_totalWakeups(0),
     m_timer(timer),
-    m_timerId(timer->timerId()),
+    m_timerId(-1),
     m_lastReceiver(0)
 {
+  if (QTimer *t = qobject_cast<QTimer*>(timer)) {
+    m_type = QTimerType;
+    m_timerId = t->timerId();
+  }
 }
 
 TimerInfo::TimerInfo(int timerId)
@@ -47,6 +56,11 @@ TimerInfo::TimerInfo(int timerId)
 {
 }
 
+TimerInfo::Type TimerInfo::type() const
+{
+  return m_type;
+}
+
 void TimerInfo::addEvent(const TimeoutEvent &timeoutEvent)
 {
   m_timeoutEvents.append(timeoutEvent);
@@ -59,11 +73,18 @@ int TimerInfo::numEvents() const
   return m_timeoutEvents.size();
 }
 
-QTimer *TimerInfo::timer() const
+QObject* TimerInfo::timerObject() const
 {
   return m_timer;
 }
 
+QTimer *TimerInfo::timer() const
+{
+  if (m_type != QTimerType)
+    return 0;
+  return qobject_cast<QTimer*>(m_timer);
+}
+
 int TimerInfo::timerId() const
 {
   return m_timerId;
@@ -95,13 +116,13 @@ QString TimerInfo::wakeupsPerSec() const
     const float wakeupsPerSec = totalWakeups / (float)timeSpan * 1000.0f;
     return QString::number(wakeupsPerSec, 'f', 1);
   }
-  return "0";
+  return QStringLiteral("0");
 }
 
 QString TimerInfo::timePerWakeup() const
 {
   if (m_type == QObjectType) {
-    return "N/A";
+    return QStringLiteral("N/A");
   }
 
   int totalWakeups = 0;
@@ -118,13 +139,13 @@ QString TimerInfo::timePerWakeup() const
   if (totalWakeups > 0) {
     return QString::number(totalTime / (float)totalWakeups, 'f', 1);
   }
-  return "N/A";
+  return QStringLiteral("N/A");
 }
 
 QString TimerInfo::maxWakeupTime() const
 {
   if (m_type == QObjectType) {
-    return "N/A";
+    return QStringLiteral("N/A");
   }
 
   int max = 0;
@@ -144,19 +165,36 @@ int TimerInfo::totalWakeups() const
 
 QString TimerInfo::state() const
 {
-  if (!m_timer){
-    return QObject::tr("None");
+  switch (type()) {
+    case QTimerType:
+    {
+      const QTimer *t = timer();
+      if (!t)
+        return QObject::tr("None");
+      if (!t->isActive())
+        return QObject::tr("Inactive");
+      if (t->isSingleShot())
+        return QObject::tr("Singleshot (%1 ms)").arg(t->interval());
+      return QObject::tr("Repeating (%1 ms)").arg(t->interval());
+    }
+    case QQmlTimerType:
+    {
+      const QObject *obj = timerObject();
+      if (!obj)
+        return QObject::tr("None");
+      const int interval = obj->property("interval").toInt();
+      if (!obj->property("running").toBool())
+        return QObject::tr("Inactive (%1 ms)").arg(interval);
+      if (obj->property("repeat").toBool())
+        return QObject::tr("Repeating (%1 ms)").arg(interval);
+      return QObject::tr("Singleshot (%1 ms)").arg(interval);
+    }
+    case QObjectType:
+      return QStringLiteral("N/A");
   }
 
-  if (!m_timer->isActive()) {
-    return QObject::tr("Inactive");
-  } else {
-    if (m_timer->isSingleShot()) {
-      return QObject::tr("Singleshot (%1 ms)").arg(m_timer->interval());
-    } else {
-      return QObject::tr("Repeating (%1 ms)").arg(m_timer->interval());
-    }
-  }
+  Q_ASSERT(false);
+  return QString();
 }
 
 void TimerInfo::removeOldEvents()
@@ -173,13 +211,18 @@ void TimerInfo::setLastReceiver(QObject *receiver)
 
 QString TimerInfo::displayName() const
 {
-  if (timer()) {
-    return Util::displayString(timer());
-  } else {
-    if (m_lastReceiver) {
-      return Util::displayString(m_lastReceiver);
-    } else {
-      return QObject::tr("Unknown QObject");
-    }
+  switch (m_type) {
+    case QTimerType:
+    case QQmlTimerType:
+      return Util::displayString(timerObject());
+    case QObjectType:
+      if (m_lastReceiver) {
+        return Util::displayString(m_lastReceiver);
+      } else {
+        return QObject::tr("Unknown QObject");
+      }
   }
+
+  Q_ASSERT(false);
+  return QString();
 }
