@@ -4,9 +4,14 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2010-2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Copyright (C) 2010-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Volker Krause <volker.krause@kdab.com>
 
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  accordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
@@ -21,6 +26,8 @@
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include <config-gammaray.h>
+
 #include "webinspector.h"
 #include "webviewmodel.h"
 
@@ -28,10 +35,15 @@
 #include <core/objecttypefilterproxymodel.h>
 #include <core/probeinterface.h>
 #include <core/singlecolumnobjectproxymodel.h>
-#include <core/probesettings.h>
 
+#include <QDebug>
+#include <QUrl>
 #include <QtPlugin>
 
+#ifdef HAVE_QT_WEBKIT1
+#include <QWebPage>
+#endif
+
 using namespace GammaRay;
 
 WebInspector::WebInspector(ProbeInterface *probe, QObject *parent)
@@ -39,11 +51,14 @@ WebInspector::WebInspector(ProbeInterface *probe, QObject *parent)
 {
   WebViewModel *webViewModel = new WebViewModel(this);
   webViewModel->setSourceModel(probe->objectListModel());
-  probe->registerModel("com.kdab.GammaRay.WebPages", webViewModel);
+  probe->registerModel(QStringLiteral("com.kdab.GammaRay.WebPages"), webViewModel);
 
   connect(probe->probe(), SIGNAL(objectCreated(QObject*)), SLOT(objectAdded(QObject*)));
 
-  const QString serverAddress = ProbeSettings::value("TCPServer", QLatin1String("0.0.0.0")).toString();
+  const QUrl serverUrl = Endpoint::instance()->serverAddress();
+  QString serverAddress(QStringLiteral("0.0.0.0"));
+  if (serverUrl.scheme() == QLatin1String("tcp"))
+    serverAddress = serverUrl.host();
   qputenv("QTWEBKIT_INSPECTOR_SERVER", serverAddress.toLocal8Bit() + ':' + QByteArray::number(Endpoint::defaultPort() + 1));
 }
 
@@ -88,9 +103,9 @@ QStringList WebInspectorFactory::supportedTypes() const
 {
   QStringList types;
 #ifdef HAVE_QT_WEBKIT1
-  types.push_back(QWebPage::staticMetaObject.className();
+  types.push_back(QWebPage::staticMetaObject.className());
 #endif
-  types.push_back("QQuickWebView");
+  types.push_back(QStringLiteral("QQuickWebView"));
   return types;
 }
 
