@@ -4,9 +4,14 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2010-2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Copyright (C) 2010-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Volker Krause <volker.krause@kdab.com>
 
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  accordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
@@ -39,6 +44,9 @@ class MainWindow : public QMainWindow
     explicit MainWindow(QWidget *parent = 0);
     ~MainWindow();
 
+  signals:
+    void targetQuitRequested();
+
   private slots:
     void about();
     void aboutPlugins();
@@ -49,6 +57,8 @@ class MainWindow : public QMainWindow
 
     void quitHost();
     void detachProbe();
+    void navigateToCode(const QString &filePath, int lineNumber, int columnNumber);
+    void setCodeNavigationIDE(QAction *action);
 
   private:
     QWidget* createErrorPage(const QModelIndex &index);
