@@ -4,9 +4,14 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2011-2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Copyright (C) 2011-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Volker Krause <volker.krause@kdab.com>
 
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  accordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
@@ -34,9 +39,9 @@ PropertyColorEditor::PropertyColorEditor(QWidget *parent)
 
 void PropertyColorEditor::edit()
 {
-  const QColor color = QColorDialog::getColor(value().value<QColor>(), this);
+  const QColor color = QColorDialog::getColor(value().value<QColor>(), this, QString(), QColorDialog::ShowAlphaChannel);
   if (color.isValid()) {
-    setValue(QVariant::fromValue(color));
+    save(QVariant::fromValue(color));
   }
 }
 
