@@ -4,9 +4,14 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2010-2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Copyright (C) 2010-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Volker Krause <volker.krause@kdab.com>
 
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  accordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
@@ -52,7 +57,7 @@ class GAMMARAY_UI_EXPORT PropertyWidget : public QTabWidget
     QString objectBaseName() const;
     void setObjectBaseName(const QString &baseName);
 
-    template<typename T> static void registerTab(QString name, QString label)
+    template<typename T> static void registerTab(const QString &name, const QString &label)
     {
       s_tabFactories << new PropertyWidgetTabFactory<T>(name, label);
       foreach (PropertyWidget *widget, s_propertyWidgets)
@@ -61,16 +66,19 @@ class GAMMARAY_UI_EXPORT PropertyWidget : public QTabWidget
 
   private:
     void createWidgets();
+    bool extensionAvailable(PropertyWidgetTabFactoryBase *factory) const;
 
   private slots:
-    void updateShownTabs(const QStringList& availableExtensions);
+    void updateShownTabs();
 
 
   private:
     QString m_objectBaseName;
 
-    // Contains all tab widgets
-    QHash<PropertyWidgetTabFactoryBase*, QWidget*> m_tabWidgets;
+    // Contains all tab widgets we have instantiated and their corresponding factories
+    // order matters, therefore these are two vectors rather than a hash or map
+    QVector<PropertyWidgetTabFactoryBase*> m_usedFactories;
+    QVector<QWidget*> m_tabWidgets;
 
     PropertyControllerInterface *m_controller;
 
