@@ -2,9 +2,14 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2011-2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Copyright (C) 2011-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Volker Krause <volker.krause@kdab.com>
 
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  accordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
@@ -33,21 +38,19 @@ namespace GammaRay {
  */
 class ProxyToolUiFactory : public ProxyFactory<ToolUiFactory>
 {
+  Q_OBJECT
   public:
     /**
      * @param path Path to the plugin spec file
      */
-    explicit ProxyToolUiFactory(const QString &path, QObject *parent = 0);
+    explicit ProxyToolUiFactory(const PluginInfo &pluginInfo, QObject *parent = 0);
 
     /** Returns @c true if the plugin seems valid from all the information we have so far. */
     bool isValid() const;
 
-    /*override*/ bool remotingSupported() const;
-    /*override*/ QWidget *createWidget(QWidget *parentWidget);
-    void initUi();
-
-private:
-  bool m_remotingSupported;
+    bool remotingSupported() const Q_DECL_OVERRIDE;
+    QWidget *createWidget(QWidget *parentWidget) Q_DECL_OVERRIDE;
+    void initUi() Q_DECL_OVERRIDE;
 };
 
 }
