@@ -4,9 +4,14 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2010-2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Copyright (C) 2010-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Milian Wolff <milian.wolff@kdab.com>
 
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  accordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
@@ -33,7 +38,7 @@ namespace GammaRay {
 void showSplashScreen()
 {
   if (!splash) {
-    QPixmap pixmap(QLatin1String(":gammaray/splashscreen.png"));
+    QPixmap pixmap(QStringLiteral(":gammaray/splashscreen.png"));
     splash = new QSplashScreen(pixmap);
     splash->setMask(pixmap.mask());
   }
