@@ -2,9 +2,14 @@
   This file is part of GammaRay, the Qt application inspection and
   manipulation tool.
 
-  Copyright (C) 2010-2014 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
+  Copyright (C) 2010-2015 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
   Author: Volker Krause <volker.krause@kdab.com>
 
+  Licensees holding valid commercial KDAB GammaRay licenses may use this file in
+  accordance with GammaRay Commercial License Agreement provided with the Software.
+
+  Contact info@kdab.com if any conditions of this licensing are not clear to you.
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
@@ -75,17 +80,17 @@ template <typename ToolUi>
 class StandardToolUiFactory : public ToolUiFactory
 {
 public:
-  virtual inline QString id() const
+  QString id() const Q_DECL_OVERRIDE
   {
     return QString(); // TODO is this a problem??
   }
 
-  virtual inline QWidget *createWidget(QWidget *parentWidget)
+  QWidget *createWidget(QWidget *parentWidget) Q_DECL_OVERRIDE
   {
     return new ToolUi(parentWidget);
   }
 
-  virtual bool remotingSupported() const
+  bool remotingSupported() const Q_DECL_OVERRIDE
   {
     return true;
   }
